/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.ldap;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.logging.Logger;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.backend.data.HashMapVascBackendAccessDataRecord;

import com.novell.ldap.LDAPAttribute;
import com.novell.ldap.LDAPConnection;
import com.novell.ldap.LDAPEntry;
import com.novell.ldap.LDAPException;
import com.novell.ldap.LDAPModification;
import com.novell.ldap.LDAPSearchConstraints;
import com.novell.ldap.LDAPSearchResults;

/**
 * LdapVascBackend provides abstract data access for ldap.
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 4, 2008
 */
public class LdapVascBackendCrud extends AbstractLdapVascBackend implements VascBackendCrud<HashMap<String,Serializable>,HashMap<String,Serializable>> {
	
	private Logger logger = Logger.getLogger(LdapVascBackendCrud.class.getName());
	private String keyAttribute = null;
	private String ldapFilter = null;
	private String createObjectClass = null;
	private int timeLimit = 10000;
	private boolean referralFollowing = true;
	
	@Override
	protected void startBackendLdap() {
	}
	
	@Override
	public VascBackendAccessDataRecord<HashMap<String, Serializable>, HashMap<String, Serializable>> createVascBackendAccessDataRecord() {
		return new HashMapVascBackendAccessDataRecord(getKeyMapType());
	}

	@Override
	public HashMap<String, Serializable> fetch(HashMap<String, Serializable> recordPK) throws VascBackendException {
		return null;
	}

	@Override
	public HashMap<String, Serializable> newRecord() throws VascBackendException {
		return new HashMap<String,Serializable>();
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#merge(java.lang.Object)
	 */
	@Override
	public HashMap<String,Serializable> merge(HashMap<String,Serializable> map) throws VascBackendException {
		LdapConnectionProvider prov = getLdapConnectionProvider();
		LDAPConnection connection = prov.getLdapConnection();
		try {
			String keyValue = (String)map.get(keyAttribute);
			LDAPSearchConstraints cons = new LDAPSearchConstraints();
			cons.setBatchSize( 0 );
			cons.setTimeLimit(getTimeLimit() ) ;
			cons.setReferralFollowing(isReferralFollowing());
			connection.setConstraints(cons);
			int searchScope = LDAPConnection.SCOPE_ONE;
			String searchBase  = getBaseDN();
			String filter = "(&"+ldapFilter+"("+keyAttribute+"="+keyValue+"))";
			LDAPSearchResults searchResults = connection.search(
					searchBase,      // object to read
					searchScope,  // scope - read single object
					filter,   // search filter
					null,         // return all attributes
					false);       // return attrs and values
			
			if (searchResults.hasMore()==false) {
				// no result to mod
				return map;
			}
			LDAPEntry entry = searchResults.next();
			List<LDAPModification> mods = new ArrayList<LDAPModification>(20);
			for (String key:map.keySet()) {
				Object value = map.get(key);
				LDAPAttribute attr = entry.getAttribute(key);
				if (attr==null) {
					LDAPModification mod = new LDAPModification(LDAPModification.ADD,new LDAPAttribute(key,(String)value));
					mods.add(mod);
					continue;
				}
				String[] s = attr.getStringValueArray();
				if (s.length==1) {
					String v = (String)value;
					if (attr.getStringValue().equals(v)==false) {
						LDAPModification mod = new LDAPModification(LDAPModification.REPLACE,new LDAPAttribute(key,v));
						mods.add(mod);
					}
					map.put(attr.getName(), attr.getStringValue());
				} else {
					List<String> multiValue = new ArrayList<String>(s.length);
					for (String ss:s) {
						multiValue.add(ss);
					}
					List<String> v = null;
					if (value instanceof String) {
						v = new ArrayList<String>(1);
						v.add((String)value);
					} else {
						v = (List<String>)value;
					}
					if (v.equals(multiValue)==false) {
						LDAPAttribute a = new LDAPAttribute(key);
						for (String vv:v) {
							a.addValue(vv);	
						}
						LDAPModification mod = new LDAPModification(LDAPModification.REPLACE,a);
						mods.add(mod);
					}
				}
			}
			
			LDAPModification[] m = new LDAPModification[mods.size()];
			m = mods.toArray(m);
			connection.modify(entry.getDN(), m);
			
			return map;
		} catch (LDAPException e) {
			throw new VascBackendException(e);
		} finally {
			safeDisconnect(connection);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#persist(java.lang.Object)
	 */
	@Override
	public HashMap<String,Serializable> persist(HashMap<String,Serializable> map) throws VascBackendException {
		LdapConnectionProvider prov = getLdapConnectionProvider();
		LDAPConnection connection = prov.getLdapConnection();
		try {
			String keyValue = (String)map.get(keyAttribute);
			LDAPEntry entry = new LDAPEntry(keyAttribute+"="+keyValue+","+getBaseDN());
			
			LDAPAttribute ob = new LDAPAttribute("objectClass");
			String[] obs = createObjectClass.split(",");
			for (String o:obs) {
				ob.addValue(o);
			}
			entry.getAttributeSet().add(ob);
			
			for (String key:map.keySet()) {
				Object value = map.get(key);
				if (value==null) {
					continue;
				}
				LDAPAttribute attr = new LDAPAttribute(key);
				attr.addValue(""+value);
				entry.getAttributeSet().add(attr);
			}
			connection.add(entry);
		} catch (LDAPException e) {
			throw new VascBackendException(e);
		} finally {
			safeDisconnect(connection);
		}
		return fetch(map);
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#delete(java.lang.Object)
	 */
	@Override
	public void delete(HashMap<String,Serializable> map) throws VascBackendException {
		LdapConnectionProvider prov = getLdapConnectionProvider();
		LDAPConnection connection = prov.getLdapConnection();
		try {
			String keyValue = (String)map.get(keyAttribute);
			int searchScope = LDAPConnection.SCOPE_ONE;
			String searchBase  = getBaseDN();
			String filter = "(&"+ldapFilter+"("+keyAttribute+"="+keyValue+"))";
			LDAPSearchResults searchResults = connection.search(
					searchBase,      // object to read
					searchScope,  // scope - read single object
					filter,   // search filter
					null,         // return all attributes
					false);       // return attrs and values
			
			if (searchResults.hasMore()==false) {
				// no result to mod
				return;
			}
			LDAPEntry entry = searchResults.next();
			connection.delete(entry.getDN());
		} catch (LDAPException e) {
			throw new VascBackendException(e);
		} finally {
			safeDisconnect(connection);
		}
	}
	
	/**
	 * @return the keyAttribute.
	 */
	public String getKeyAttribute() {
		return keyAttribute;
	}
	
	/**
	 * @param keyAttribute the keyAttribute to set.
	 */
	public void setKeyAttribute(String keyAttribute) {
		this.keyAttribute = keyAttribute;
	}
	
	/**
	 * @return the ldapFilter.
	 */
	public String getLdapFilter() {
		return ldapFilter;
	}
	
	/**
	 * @param ldapFilter the ldapFilter to set.
	 */
	public void setLdapFilter(String ldapFilter) {
		this.ldapFilter = ldapFilter;
	}
	
	/**
	 * @return the createObjectClass.
	 */
	public String getCreateObjectClass() {
		return createObjectClass;
	}
	
	/**
	 * @param createObjectClass the createObjectClass to set.
	 */
	public void setCreateObjectClass(String createObjectClass) {
		this.createObjectClass = createObjectClass;
	}
	
	/**
	 * @return the timeLimit.
	 */
	public int getTimeLimit() {
		return timeLimit;
	}
	
	/**
	 * @param timeLimit the timeLimit to set.
	 */
	public void setTimeLimit(int timeLimit) {
		this.timeLimit = timeLimit;
	}
	
	/**
	 * @return the referralFollowing.
	 */
	public boolean isReferralFollowing() {
		return referralFollowing;
	}
	
	/**
	 * @param referralFollowing the referralFollowing to set.
	 */
	public void setReferralFollowing(boolean referralFollowing) {
		this.referralFollowing = referralFollowing;
	}
}
