/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.ldap;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.EnumSet;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.logging.Logger;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.backend.data.HashMapVascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.list.DefaultVascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendList;
import net.forwardfire.vasc.backend.list.VascBackendListFeature;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;

import com.novell.ldap.LDAPAttribute;
import com.novell.ldap.LDAPAttributeSet;
import com.novell.ldap.LDAPConnection;
import com.novell.ldap.LDAPEntry;
import com.novell.ldap.LDAPException;
import com.novell.ldap.LDAPModification;
import com.novell.ldap.LDAPSearchConstraints;
import com.novell.ldap.LDAPSearchResults;

/**
 * LdapVascBackend provides abstract data access for ldap.
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 4, 2008
 */
public class LdapVascBackendList extends AbstractLdapVascBackend implements VascBackendList<HashMap<String,Serializable>,HashMap<String,Serializable>> {
	
	private Logger logger = Logger.getLogger(LdapVascBackendList.class.getName());
	private final EnumSet<VascBackendListFeature> features = EnumSet.noneOf(VascBackendListFeature.class);
	private final List<String> rowActions = new ArrayList<String>();
	private String keyAttribute = null;
	private String ldapFilter = null;
	private String createObjectClass = null;
	private int timeLimit = 10000;
	private boolean referralFollowing = true;
	
	@Override
	protected void startBackendLdap() {
		// TODO Auto-generated method stub
		
	}
	
	@Override
	public VascBackendAccessDataRecord<HashMap<String, Serializable>, HashMap<String, Serializable>> createVascBackendAccessDataRecord() {
		return new HashMapVascBackendAccessDataRecord(getKeyMapType());
	}
	
	@Override
	public EnumSet<VascBackendListFeature> getSupportedFeatures() {
		return features;
	}
	
	@Override
	public List<String> getSupportedRowActions() {
		return rowActions;
	}
	
	@Override
	public void executeRowAction(VascBackendListRequest request,List<HashMap<String, Serializable>> recordPKs, String actionName) throws VascBackendException {
		
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.crud.VascBackendCrud#execute()
	 */
	@Override
	public VascBackendListResult<HashMap<String,Serializable>> execute(VascBackendListRequest state) throws VascBackendException {
		LdapConnectionProvider prov = getLdapConnectionProvider();
		LDAPConnection connection = prov.getLdapConnection();
		List<HashMap<String,Serializable>> result = new ArrayList<HashMap<String,Serializable>>(50); 
		try {
			
			LDAPSearchConstraints cons = new LDAPSearchConstraints();
			cons.setBatchSize( 0 );
			cons.setTimeLimit(getTimeLimit() ) ;
			cons.setReferralFollowing(isReferralFollowing());
			connection.setConstraints(cons);

			int searchScope = LDAPConnection.SCOPE_ONE;
			String searchBase  = getBaseDN();
			
			LDAPSearchResults searchResults = connection.search(
					searchBase,       // object to read
					searchScope,      // scope - read single object
					getLdapFilter(),  // search filter
					null,             // return all attributes
					false);           // return attrs and values
			
			while (searchResults.hasMore()) {
				LDAPEntry entry = searchResults.next();
				HashMap<String,Serializable> map = new HashMap<String,Serializable>(10);
				
				LDAPAttributeSet attributeSet = entry.getAttributeSet();
				Iterator<LDAPAttribute> i = attributeSet.iterator();
				while (i.hasNext()) {
					LDAPAttribute attr = i.next();
					String[] s = attr.getStringValueArray();
					if (s.length==1) {
						map.put(attr.getName(), attr.getStringValue());
					} else {
						List<String> multiValue = new ArrayList<String>(s.length);
						for (String ss:s) {
							multiValue.add(ss);
						}
						map.put(attr.getName(), (Serializable)multiValue );
					}
				}
				result.add(map);
			}
		} catch (LDAPException e) {
			throw new VascBackendException(e);
		} finally {
			safeDisconnect(connection);
		}
		return new DefaultVascBackendListResult<HashMap<String,Serializable>>(result);
	}
	
	/**
	 * @return the keyAttribute.
	 */
	public String getKeyAttribute() {
		return keyAttribute;
	}
	
	/**
	 * @param keyAttribute the keyAttribute to set.
	 */
	public void setKeyAttribute(String keyAttribute) {
		this.keyAttribute = keyAttribute;
	}
	
	/**
	 * @return the ldapFilter.
	 */
	public String getLdapFilter() {
		return ldapFilter;
	}
	
	/**
	 * @param ldapFilter the ldapFilter to set.
	 */
	public void setLdapFilter(String ldapFilter) {
		this.ldapFilter = ldapFilter;
	}
	
	/**
	 * @return the createObjectClass.
	 */
	public String getCreateObjectClass() {
		return createObjectClass;
	}
	
	/**
	 * @param createObjectClass the createObjectClass to set.
	 */
	public void setCreateObjectClass(String createObjectClass) {
		this.createObjectClass = createObjectClass;
	}
	
	/**
	 * @return the timeLimit.
	 */
	public int getTimeLimit() {
		return timeLimit;
	}
	
	/**
	 * @param timeLimit the timeLimit to set.
	 */
	public void setTimeLimit(int timeLimit) {
		this.timeLimit = timeLimit;
	}
	
	/**
	 * @return the referralFollowing.
	 */
	public boolean isReferralFollowing() {
		return referralFollowing;
	}
	
	/**
	 * @param referralFollowing the referralFollowing to set.
	 */
	public void setReferralFollowing(boolean referralFollowing) {
		this.referralFollowing = referralFollowing;
	}
}
