package net.forwardfire.vasc.backend.metamodel.bundle;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Locale;
import java.util.Properties;
import java.util.ResourceBundle;

import org.eobjects.metamodel.MetaModelException;
import org.eobjects.metamodel.QueryPostprocessDataContext;
import org.eobjects.metamodel.UpdateScript;
import org.eobjects.metamodel.UpdateableDataContext;
import org.eobjects.metamodel.data.AbstractDataSet;
import org.eobjects.metamodel.data.DataSet;
import org.eobjects.metamodel.data.DefaultRow;
import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.query.FilterItem;
import org.eobjects.metamodel.query.SelectItem;
import org.eobjects.metamodel.schema.Column;
import org.eobjects.metamodel.schema.MutableColumn;
import org.eobjects.metamodel.schema.MutableSchema;
import org.eobjects.metamodel.schema.MutableTable;
import org.eobjects.metamodel.schema.Schema;
import org.eobjects.metamodel.schema.Table;
import org.eobjects.metamodel.schema.TableType;

/**
 * ResourceBundleDataContext is updatable context for file based data source.
 * Will try to load all supported locals by jvm as tables. (except for URL)
 * 
 * @author Willem Cazander
 * @version 1.0 May 26, 2012
 */
public class ResourceBundleDataContext extends QueryPostprocessDataContext implements UpdateableDataContext {

	protected static final String DEFAULT_CHAR_SET = "UTF-8";
	protected ResourceBundleUpdateExecutor resourceBundleUpdateExecutor = null;
	protected ResourceBundleDataControl bundleDataControl = null;
	protected String mainSchemaName = null;
	protected Charset charSet = null;
	
	private ResourceBundleDataContext(Charset charSet) {
		if (charSet==null) {
			throw new NullPointerException("Can't start with null charSet");
		}
		this.charSet = charSet;
	}
	
	public ResourceBundleDataContext(String resourceBundle) {
		this(resourceBundle,Charset.forName(DEFAULT_CHAR_SET));
	}
	
	public ResourceBundleDataContext(String resourceBundle,Charset charSet) {
		this(charSet);
		if (resourceBundle==null) {
			throw new NullPointerException("Can't start with null resourceBundle");
		}
		init(resourceBundle);
	}
	
	public ResourceBundleDataContext(File resourceBundle) {
		this(resourceBundle,Charset.forName(DEFAULT_CHAR_SET));
	}
	
	public ResourceBundleDataContext(File resourceBundle,Charset charSet) {
		this(charSet);
		if (resourceBundle==null) {
			throw new NullPointerException("Can't start with null resourceBundle");
		}
		init(resourceBundle);
	}
	
	public ResourceBundleDataContext(URL resourceBundle) {
		this(resourceBundle,Charset.forName(DEFAULT_CHAR_SET));
	}
	
	public ResourceBundleDataContext(URL resourceBundle,Charset charSet) {
		this(charSet);
		if (resourceBundle==null) {
			throw new NullPointerException("Can't start with null resourceBundle");
		}
		init(resourceBundle);
	}
	
	/**
	 * Init ResourceBundleDataContext to use diffect source backends.
	 */
	protected void init(Object source) {
		if (source==null) {
			throw new NullPointerException("Can't init with null source");
		}
		if (bundleDataControl!=null) {
			throw new NullPointerException("Only one init allowed."); // can't override refreshSchema to reinit again.
		}

		// Setup data source
		String bundleBaseName = null;
		if (source instanceof File) {
			File baseFile = (File)source;
			bundleBaseName = baseFile.getName();
			bundleDataControl = new ResourceBundleDataControl(charSet,baseFile);
		} else if (source instanceof URL) {
			URL baseUrl = (URL)source;
			bundleBaseName = baseUrl.getFile();
			bundleDataControl = new ResourceBundleDataControl(charSet,baseUrl);			
		} else {
			bundleBaseName = source.toString();
			bundleDataControl = new ResourceBundleDataControl(charSet);
		}
		mainSchemaName = cleanMainSchemaName(bundleBaseName);
		
		// Load all data
		//ResourceBundle.getBundle(bundleBaseName,bundleDataControl);
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if (cl==null) { cl=getClass().getClassLoader(); }
		try {
			bundleDataControl.newBundle(bundleBaseName, Locale.getDefault(), null, cl, false);
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		// Start Query executor
		resourceBundleUpdateExecutor = new ResourceBundleUpdateExecutor(this,bundleDataControl);
	}
	
	// TODO check where static verion is.
	protected String cleanMainSchemaName(String name) {
		if (name.endsWith(ResourceBundleDataControl.BUNDLE_EXTENSION)) {
			name = name.substring(0,name.indexOf(ResourceBundleDataControl.BUNDLE_EXTENSION)-1);
		}
		StringBuffer buffer = new StringBuffer(name.length());
		for (char c:name.toCharArray()) {
			if (Character.isDigit(c)) {
				buffer.append(c);
			} else if (Character.isLetter(c)) {
				buffer.append(c);
			} else {
				buffer.append('_');
			}
		}
		return buffer.toString();
	}
	
	public boolean isDataContextWritable() {
		return bundleDataControl.isWritable();
	}
	
	protected MutableTable createTable(Schema schema,String name) {
		MutableTable table = new MutableTable(name, TableType.TABLE,	schema);
		
		MutableColumn idColumn = new MutableColumn("id");
		//idColumn.setColumnNumber(0);
		idColumn.setIndexed(true);
		idColumn.setNullable(false);
		//idColumn.setPrimaryKey(true);

		MutableColumn keyColumn = new MutableColumn("key");
		keyColumn.setColumnNumber(1);
		keyColumn.setNullable(false);
		
		MutableColumn valueColumn = new MutableColumn("value");
		valueColumn.setColumnNumber(2);
		valueColumn.setNullable(false);
		
		table.addColumn(idColumn);
		table.addColumn(keyColumn);
		table.addColumn(valueColumn);
		
		return table;
	}
	
	// ======== Interface

	
	@Override
    protected Number executeCountQuery(Table table, List<FilterItem> whereItems, boolean functionApproximationAllowed) {
        return null;
    }
	
	@Override
	protected Schema getMainSchema() throws MetaModelException {		
		String schemaName = getDefaultSchemaName();
		MutableSchema schema = new MutableSchema(schemaName);
		for (String bundleName:bundleDataControl.getBundleNames()) {
			schema.addTable(createTable(schema,bundleName));	
		}
		return schema;
	}

	// TODO: move to SelectItem
	static public SelectItem[] convertColumns(Column[] cols) {
		SelectItem[] selectItems = new SelectItem[cols.length];
		for (int i=0;i<cols.length;i++) {
			selectItems[i] = new SelectItem(cols[i]);
		}
		return selectItems;
	}
	
	@Override
	protected DataSet materializeMainSchemaTable(Table table, Column[] cols,int maxRows) {
		return materializeTable(table.getName(),convertColumns(table.getColumns()),maxRows);
	}

	protected DataSet materializeTable(String table,final SelectItem[] selectItems,final int maxRows) {
		Properties data = bundleDataControl.getBundleData(table);
		List<Row> result = new ArrayList<Row>(200);
		for (Object keyO:data.keySet()) {
			String keyId = Integer.toString(keyO.hashCode());
			String key = (String)keyO;
			String value = data.getProperty(key);
			result.add(new DefaultRow(selectItems, new Object[] {keyId,key,value}));
		}
		final Iterator<Row> resultData = result.iterator();
		return new AbstractDataSet() {
			int resultCount = 0;
			@Override
			public SelectItem[] getSelectItems() {	return selectItems;				}
			@Override
			public Row getRow() {					return resultData.next();		}
			@Override
			public boolean next() {
				if (maxRows > 0 && resultCount++ > maxRows) {
					return false;
				}
				return resultData.hasNext();
			}
		};
	}
	
	@Override
	public void executeUpdate(UpdateScript update) {
		update.run(resourceBundleUpdateExecutor);
	}

	@Override
	protected String getMainSchemaName() throws MetaModelException {
		return mainSchemaName;
	}
}
