package net.forwardfire.vasc.backend.metamodel.bundle;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.Charset;
import java.util.Collections;
import java.util.Date;
import java.util.HashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.PropertyResourceBundle;
import java.util.ResourceBundle;
import java.util.ResourceBundle.Control;
import java.util.Set;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * ResourceBundleControl load ResourceBundles with correct CharSet from file and class path.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 26, 2012
 */
public class ResourceBundleDataControl extends Control {
	
	/** The default extension for bundles. */
	protected static final String BUNDLE_EXTENSION = "properties";
	protected static final String BUNDLE_COMMENT = "Saved by MetaModel; ";
	
	protected Logger logger = null;
	protected Charset charSet = null;
	
	/** The charSet to load with **/
	protected File baseBundleFile = null;
	protected URL baseBundleUrl = null;
	
	protected Map<String,Properties> bundleData = null;
	protected Map<String,File> bundleFiles = null;
	
	private ResourceBundleDataControl() {
		logger = LoggerFactory.getLogger(ResourceBundleDataControl.class);
		bundleFiles = new HashMap<String,File>(10);
		bundleData = Collections.synchronizedMap(new HashMap<String,Properties>(10));
	}
	
	/**
	 * Creates ResourceBundleControl to read ResourceBundle with charSet.
	 */
	public ResourceBundleDataControl(Charset charSet) {
		this();
		if (charSet==null) {
			throw new NullPointerException("Can't load bundle with null Charset.");
		}
		this.charSet=charSet;
	}
	
	public ResourceBundleDataControl(Charset charSet,File baseBundleFile) {
		this(charSet);
		if (baseBundleFile==null) {
			throw new NullPointerException("Can't load bundle with null baseBundleFile.");
		}
		this.baseBundleFile=baseBundleFile;
	}
	
	public ResourceBundleDataControl(Charset charSet,URL baseBundleUrl) {
		this(charSet);
		if (baseBundleUrl==null) {
			throw new NullPointerException("Can't load bundle with null baseBundleUrl.");
		}
		this.baseBundleUrl=baseBundleUrl;
	}
	
	public boolean isWritable() {
		return baseBundleFile!=null;
	}
	
	public Set<String> getBundleNames() {
		return bundleData.keySet();
	}
	
	public Properties getBundleData(String bundleName) {
		if (bundleName==null) {
			throw new NullPointerException("Can't get data for null bundleName.");
		}
		return bundleData.get(bundleName);
	}
	
	public void removeBundle(String bundleName) {
		Properties data = getBundleData(bundleName);
		if (data==null) {
			throw new IllegalStateException("Can't remove bundle data of unknown bundle: "+bundleName);
		}
		bundleData.remove(bundleName);
		bundleFiles.remove(bundleName);
	}
	
	public void saveBundle(String bundleName) throws IOException {
		Properties data = getBundleData(bundleName);
		if (data==null) {
			throw new IllegalStateException("Can't save bundle data of unknown bundle: "+bundleName);
		}
		File bundleFile = bundleFiles.get(bundleName);
		if (bundleFile==null) {
			throw new IllegalStateException("Could not find loaded bundle file: "+bundleName);
		}
		long startTime = System.currentTimeMillis();
		saveBundleFile(data,bundleFile);
		long stopTime = System.currentTimeMillis();
		logger.info("Saved bundle "+bundleName+" with "+data.size()+" keys in "+(stopTime-startTime)+" ms.");
	}
	
	public int putBundleData(ResourceBundle bundle,String bundleName,File file) {
		Properties p = new Properties();
		for (String key:bundle.keySet()) {
			p.put(key, bundle.getString(key));
		}
		bundleData.put(bundleName, p);
		if (file!=null) {
			bundleFiles.put(bundleName, file);
		}
		return p.size();
	}
	
	/**
	 * Does the loading of the ResourceBundle
	 */
	@Override
    public ResourceBundle newBundle(String baseName, Locale localeRequested, String format, ClassLoader loader, boolean reload) throws IllegalAccessException, InstantiationException, IOException {
    	long startTime = System.currentTimeMillis();
    	
    	// On urls only load the requested one.
    	if (baseBundleUrl!=null) {
    		ResourceBundle bundle = loadBundleUrl(baseBundleUrl);
    		putBundleData(bundle, baseBundleUrl.getFile(), null);
    		return bundle;
    	}
    	
    	if (baseName.endsWith(BUNDLE_EXTENSION)) {
    		baseName = baseName.substring(0,baseName.indexOf(BUNDLE_EXTENSION)-1);
		}
    	
    	// Try to load all bundles.
    	ResourceBundle bundleLast = null;
    	int totalBundleDataSize = 0;
    	for (Locale locale:Locale.getAvailableLocales()) {
    		String bundleName = toBundleName(baseName, locale);
    		ResourceBundle bundle = null;
    		File bundleFile = null;
    		try {
    			if (baseBundleFile==null) {
    				String resourceName = toResourceName(bundleName, BUNDLE_EXTENSION);
    				logger.info("Loading: "+resourceName);
    	        	bundle = loadBundleClassPath(resourceName,loader,reload);
    			} else {
    				String resourceName = baseBundleFile.getParent()+File.separatorChar+bundleName+"."+BUNDLE_EXTENSION;
    				logger.info("Loading: "+resourceName);
    				bundleFile = new File(resourceName);
    				bundle = loadBundleFile(bundleFile);
    			}
    		} catch (FileNotFoundException fnfe) {
    		} catch (IOException e) {
    			e.printStackTrace();
    			continue; // next
    		}
    		if (bundle==null) {
    			continue;
    		}
    		bundleLast = bundle;
    		String tableName = toBundleName("", locale).substring(1);
    		totalBundleDataSize += putBundleData(bundle,tableName,bundleFile);
    	}
    	
    	long stopTime = System.currentTimeMillis();
		logger.info("Loaded total "+bundleData.size()+" bundles with "+totalBundleDataSize+" keys in "+(stopTime-startTime)+" ms.");
		
    	return bundleLast; // just return last one, not requested one.
    }
    
    protected ResourceBundle loadBundleClassPath(String resourceName,ClassLoader loader,boolean reload) throws IOException {
    	ResourceBundle bundle = null;
        InputStream stream = null;
        if (reload) {
            URL url = loader.getResource(resourceName);
            if (url != null) {
                URLConnection connection = url.openConnection();
                if (connection != null) {
                    connection.setUseCaches(false);
                    stream = connection.getInputStream();
                }
            }
        } else {
            stream = loader.getResourceAsStream(resourceName);
        }
        if (stream != null) {
        	bundle = loadBundleStream(stream);
        }
        return bundle;
    }
    
    protected ResourceBundle loadBundleUrl(URL url) throws IOException {
    	ResourceBundle bundle = null;
        InputStream stream = null;
        URLConnection connection = url.openConnection();
        if (connection != null) {
            connection.setUseCaches(false);
            stream = connection.getInputStream();
        }  
        if (stream != null) {
        	bundle = loadBundleStream(stream);
        }
        return bundle;
    }
    
    protected ResourceBundle loadBundleFile(File file) throws IOException {
    	ResourceBundle bundle = null;
        InputStream stream = new FileInputStream(file);
        if (stream != null) {
            bundle = loadBundleStream(stream);
        }
        return bundle;
    }
    
    protected ResourceBundle loadBundleStream(InputStream stream) throws IOException {
    	if (stream==null) {
    		return null;
    	}
    	try {
            return new PropertyResourceBundle(new InputStreamReader(stream, charSet));
        } finally {
            stream.close();
        }
    }
    
    protected void saveBundleFile(Properties properties,File file) throws IOException {
    	OutputStream out = new FileOutputStream(file);
    	try {
    		properties.store(new OutputStreamWriter(out,charSet), BUNDLE_COMMENT+new Date());
        } finally {
        	if (out!=null) {
        		out.close();
        	}
        }
    }
}
