package net.forwardfire.vasc.backend.metamodel.crud;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.eobjects.metamodel.DataContext;
import org.eobjects.metamodel.MetaModelException;
import org.eobjects.metamodel.UpdateScript;
import org.eobjects.metamodel.UpdateableDataContext;
import org.eobjects.metamodel.data.DataSet;
import org.eobjects.metamodel.data.Row;
import org.eobjects.metamodel.query.CompiledQuery;
import org.eobjects.metamodel.query.Query;
import org.eobjects.metamodel.query.builder.InitFromBuilder;
import org.eobjects.metamodel.query.builder.InitFromBuilderImpl;
import org.eobjects.metamodel.schema.Column;
import org.eobjects.metamodel.schema.ColumnType;
import org.eobjects.metamodel.schema.Schema;
import org.eobjects.metamodel.schema.Table;

/**
 * CrudDataContextImpl is implementation of CrudDataContext interface which provides crud operations on UpdateableDataContext 
 * 
 * @author Willem Cazander
 * @version 1.0 May 23, 2012
 */
public class CrudDataContextImpl extends AbstractCrudDataContext implements AbstractCrudDataContext.QueryWrapDataSetPrimaryKeyHandler {

	protected UpdateableDataContext dataContextDelegate = null;
	protected Map<String,List<String>> tableKeys = null;
	private boolean checkPrimaryKeysAllRows = false;
	
	public CrudDataContextImpl(UpdateableDataContext dataContext) {
		dataContextDelegate = dataContext;
		tableKeys = new HashMap<String,List<String>>(100);
	}
	
	public void overridePrimaryKeysForTable(String table,List<String> keys) {
		tableKeys.put(table, keys);
	}
	
	@Override
	public List<String> getPrimaryKeysForTable(Table table) {
		List<String> keys = tableKeys.get(table.getName());
		if (keys!=null) {
			return keys;
		}
		keys = new ArrayList<String>(5);
		for (Column k:table.getPrimaryKeys()) {
			keys.add(k.getName());  // strange getPrimaryKeys is mostly empty, check SimpleTableDef.toTable or MutableColumn.setType to fix
		}
		// start test hack code
		if (keys.isEmpty()) { 
			for (Column k:table.getColumnsOfType(ColumnType.ROWID)) { // mmm still empty
				keys.add(k.getName());
			}
			if (keys.isEmpty()) {
				for (Column k:table.getColumns()) {
					String colName = k.getName(); 
					if (colName.equals("_id")) {
						keys.add(colName);
					} else if (colName.equalsIgnoreCase("id")) { // TODO: rm this quick hard code ID search
						keys.add(colName);
					}
				}	
			}
		}
		// stop test hack code
		if (keys.isEmpty()) {
			throw new IllegalStateException("Can't work with table: "+table+" without primary key.");
		}
		return keys;
	}
	
	/**
	 * Does the actual automatic Row to UpdateableRow wrapping.
	 */
	@Override
	public UpdateableRow wrapOrCreateToUpdateableRow(Row row,Table table) {
		UpdateableRowMapImpl result = new UpdateableRowMapImpl(table,getPrimaryKeysForTable(table));
		if (row==null) {
			result.addSelectItems(table.getColumns()); // create new when row is null
		} else {
			result.addSelectItems(row.getSelectItems()); 
			result.setValue(row); // copy all values to impl
		}
		return result;
	}	
	
	/**
	 * Checks if backend dataContextDelegate has native support for UpdateableRowDataContext.
	 * If not then automatic wrap full DataSet data to UpdateableRow
	 */
	@Override
	public DataSet executeQuery(Query query) throws MetaModelException {
		if (dataContextDelegate instanceof UpdateableRowDataContext) {
			return ((UpdateableRowDataContext) dataContextDelegate).crudExecuteQuery(this,query);
		} else {
			return executeQueryWrapDataSet(dataContextDelegate,query,isCheckPrimaryKeysAllRows(),this);
		}
	}
		
	/**
	 * Checks if backend dataContextDelegate has native support for UpdateableRowDataContext. 
	 * If so then use api to start query builder with this datacontext.
	 * Else use default InitFromBuilderImpl query builder.
	 */
	@Override
	public InitFromBuilder query() {
		if (dataContextDelegate instanceof UpdateableRowDataContext) {
			return ((UpdateableRowDataContext) dataContextDelegate).crudCreateQuery(this);
		} else {
			return new InitFromBuilderImpl(this);
		}
	}
	
	// === Bean properties
	
	/**
	 * @return the checkPrimaryKeysAllRows
	 */
	public boolean isCheckPrimaryKeysAllRows() {
		return checkPrimaryKeysAllRows;
	}
	
	/**
	 * @param checkPrimaryKeysAllRows the checkPrimaryKeysAllRows to set
	 */
	public void setCheckPrimaryKeysAllRows(boolean checkPrimaryKeysAllRows) {
		this.checkPrimaryKeysAllRows = checkPrimaryKeysAllRows;
	}
	
	// === Start wrapper to dataContextDelegate
	
	/**
	 * @see org.eobjects.metamodel.UpdateableDataContext#executeUpdate(org.eobjects.metamodel.UpdateScript)
	 */
	@Override
	public void executeUpdate(UpdateScript arg0) {
		dataContextDelegate.executeUpdate(arg0);
	}
	
	/**
	 * @see org.eobjects.metamodel.DataContext#getColumnByQualifiedLabel(java.lang.String)
	 */
	@Override
	public Column getColumnByQualifiedLabel(String arg0) {
		return dataContextDelegate.getColumnByQualifiedLabel(arg0);
	}
	
	/**
	 * @see org.eobjects.metamodel.DataContext#getDefaultSchema()
	 */
	@Override
	public Schema getDefaultSchema() throws MetaModelException {
		return dataContextDelegate.getDefaultSchema();
	}
	
	/**
	 * @see org.eobjects.metamodel.DataContext#getSchemaByName(java.lang.String)
	 */
	@Override
	public Schema getSchemaByName(String arg0) throws MetaModelException {
		return dataContextDelegate.getSchemaByName(arg0);
	}
	
	/**
	 * @see org.eobjects.metamodel.DataContext#getSchemaNames()
	 */
	@Override
	public String[] getSchemaNames() throws MetaModelException {
		return dataContextDelegate.getSchemaNames();
	}
	
	/**
	 * @see org.eobjects.metamodel.DataContext#getSchemas()
	 */
	@Override
	public Schema[] getSchemas() throws MetaModelException {
		return dataContextDelegate.getSchemas();
	}
	
	/**
	 * @see org.eobjects.metamodel.DataContext#getTableByQualifiedLabel(java.lang.String)
	 */
	@Override
	public Table getTableByQualifiedLabel(String arg0) {
		return dataContextDelegate.getTableByQualifiedLabel(arg0);
	}
	
	/**
	 * @see org.eobjects.metamodel.DataContext#refreshSchemas()
	 */
	@Override
	public DataContext refreshSchemas() {
		return dataContextDelegate.refreshSchemas();
	}
	
	/**
	 * @see org.eobjects.metamodel.DataContext#parseQuery(java.lang.String)
	 */
	@Override
	public Query parseQuery(String queryString) throws MetaModelException {
		return dataContextDelegate.parseQuery(queryString);
	}
	
	/**
	 * @see org.eobjects.metamodel.DataContext#executeQuery(java.lang.String)
	 */
	@Override
	public DataSet executeQuery(String queryString) throws MetaModelException {
		return dataContextDelegate.executeQuery(queryString);
	}
	
	@Override
	public CompiledQuery compileQuery(Query q) throws MetaModelException {
		return dataContextDelegate.compileQuery(q);
	}
	
	@Override
	public DataSet executeQuery(CompiledQuery cq, Object... args) {
		return dataContextDelegate.executeQuery(cq, args);
	}
}
