package net.forwardfire.vasc.backend.metamodel.jndi;

import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.spi.ObjectFactory;

import net.forwardfire.vasc.backend.metamodel.jndi.loader.JndiCsvDataContextLoader;
import net.forwardfire.vasc.backend.metamodel.jndi.loader.JndiDataContextLoader;
import net.forwardfire.vasc.backend.metamodel.jndi.loader.JndiDataContextLoaderConfig;
import net.forwardfire.vasc.backend.metamodel.jndi.loader.JndiJdbcDataContextLoader;
import net.forwardfire.vasc.backend.metamodel.jndi.loader.JndiMongodbDataContextLoader;
import net.forwardfire.vasc.backend.metamodel.jndi.loader.JndiXmlDomDataContextLoader;
import net.forwardfire.vasc.backend.metamodel.jndi.loader.JndiXmlSaxDataContextLoader;

import org.eobjects.metamodel.DataContext;
import org.eobjects.metamodel.UpdateableDataContext;

/**
 * JndiDataContextObjectFactory provides data contexts on application scope level for
 * all (EE) servers which provides java naming support.
 * 
 * Tomcat context.xml config example;
 * <pre>
 * <Resource name="metamodel/databaseExampleDC" auth="Container" type="org.eobjects.metamodel.DataContext"
 *			factory="net.forwardfire.vasc.backend.metamodel.jndi.JndiDataContextObjectFactory"
 *			backendType="jdbcJndi" backendUrl="java:jdbc/databaseExampleDS"
 *		/>
 * </pre>
 * 
 * @author Willem Cazander
 * @version 1.0 May 22, 2012
 */
public class JndiDataContextObjectFactory implements ObjectFactory  {

	protected DataContext dataContext = null;
	public enum BackendType {
		jdbcJndi,jdbc,mongodb,mongodbJndi,csvFile,csvUrl,xmlDomFile,xmlDomUrl,xmlSaxFile,xmlSaxUrl,providerClass
	}
	
	/**
	 * @see javax.naming.spi.ObjectFactory#getObjectInstance(java.lang.Object, javax.naming.Name, javax.naming.Context, java.util.Hashtable)
	 */
	@Override
	synchronized public Object getObjectInstance(Object obj, Name namePath, Context nameCtx,Hashtable<?, ?> env) throws Exception {

		// Create only once.
		if (dataContext!=null) {
			return dataContext;
		}
		
		// Create objectName to print with errors
		StringBuffer buf = new StringBuffer(40);
		for (int i=0;i<namePath.size();i++) {
			buf.append(namePath.get(i));
		}
		String objectName = buf.toString(); // note: print only last part of full name path
		
		// Wrap to loader and config bean
		JndiDataContextLoaderConfig config = new JndiDataContextLoaderConfig(obj,namePath,nameCtx,env,objectName);
		BackendType type = BackendType.valueOf(config.getBackendType());
		JndiDataContextLoader loader = null;
		
		// Config loader
		switch (type) {
		default: throw new IllegalStateException("Can't handle unimplemented backendType: "+config.getBackendType());
		case jdbcJndi:		loader = new JndiJdbcDataContextLoader();			break;
		case jdbc:			loader = new JndiJdbcDataContextLoader(false);		break;
		case mongodb:		loader = new JndiMongodbDataContextLoader();		break;
		case mongodbJndi:	loader = new JndiMongodbDataContextLoader(true);	break;
		case csvFile:		loader = new JndiCsvDataContextLoader();			break;
		case csvUrl:		loader = new JndiCsvDataContextLoader(true);		break;
		case xmlDomFile:	loader = new JndiXmlDomDataContextLoader();			break;
		case xmlDomUrl:		loader = new JndiXmlDomDataContextLoader(true);		break;
		case xmlSaxFile:	loader = new JndiXmlSaxDataContextLoader();			break;
		case xmlSaxUrl:		loader = new JndiXmlSaxDataContextLoader(true);		break;
		case providerClass:
			dataContext = createDataContextProvider(objectName,config.getBackendClass()).getDataContext();
			break;
		}
		
		// Load data context except when using the providerClass type.
		if (loader!=null) {
			dataContext = loader.loadDataContext(config);
		}
		
		// Make data context read only is requested by config of factory.
		if ("true".equalsIgnoreCase(config.getBackendReadOnly()) && dataContext instanceof UpdateableDataContext) {
			dataContext = new JndiReadOnlyDataContext(dataContext);
		}
		
		// We are done lets return
		return dataContext;
	}

	/**
	 * Load DataContextProvider from class path to provide data context.
	 */
	protected DataContextProvider createDataContextProvider(String objectName,String className) {
		if (className==null) {
			throw new IllegalArgumentException("Can't work with null backendClass attribute for: "+objectName);
		}
		if (className.isEmpty()) {
			throw new IllegalArgumentException("Can't work with empty backendClass attribute for: "+objectName);
		}
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if (cl==null) {
			cl = getClass().getClassLoader();
		}
		try {
			Class<?> providerClass = cl.loadClass(className);
			DataContextProvider dataContextProvider = (DataContextProvider)providerClass.newInstance();
			return dataContextProvider;
		} catch (Exception e) {
			throw new IllegalStateException("Could not create provider of: "+className,e);
		}
	}
}
