package net.forwardfire.vasc.backend.metamodel.jndi.loader;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.RefAddr;
import javax.naming.Reference;

/**
 * JndiDataContextLoaderConfig hold generic parameters to config the loader and DataContext.
 * 
 * @author Willem Cazander
 * @version 1.0 May 22, 2012
 */
public class JndiDataContextLoaderConfig {

	public final static String BACKEND_TYPE = "backendType"; 
	public final static String BACKEND_FILE = "backendFile";
	public final static String BACKEND_URL  = "backendUrl";
	public final static String BACKEND_HOST = "backendHost";
	public final static String BACKEND_PORT = "backendPort";
	public final static String BACKEND_DB   = "backendDatabase";
	public final static String BACKEND_USER = "backendUsername";
	public final static String BACKEND_PASS = "backendPassword";
	public final static String BACKEND_CLASS= "backendClass";
	public final static String BACKEND_READ_ONLY = "backendReadOnly"; 
	
	private Object jndiObject       = null;
	private Name jndiNamePath       = null;
	private Context jndiContext     = null;
	private Hashtable<?, ?> jndiEnv = null;
	private String jndiObjectName   = null;
	private String backendType      = null;
	private String backendFile      = null;
	private String backendUrl       = null;
	private String backendHost      = null;
	private String backendPort      = null;
	private String backendDatabase  = null;
	private String backendUsername  = null;
	private String backendPassword  = null;
	private String backendClass     = null;
	private String backendReadOnly  = null;
	private Map<String,String> backendParameters = null;
	
	public JndiDataContextLoaderConfig(Object jndiObject,Name jndiNamePath,Context jndiContext,Hashtable<?, ?> jndiEnv,String jndiObjectName) {
		this.jndiObject=jndiObject;
		this.jndiNamePath=jndiNamePath;
		this.jndiContext=jndiContext;
		this.jndiEnv=jndiEnv;
		this.jndiObjectName=jndiObjectName;
		
		initBackendParameters();
		initBeanParameters();
	}
	
	protected void initBackendParameters() {
		// Copy config parameters of this factory instance to map
		backendParameters = new HashMap<String,String>(30);
		Reference ref = (Reference) getJndiObject();
		Enumeration<?> addrs = ref.getAll();
		while (addrs.hasMoreElements()) {
			RefAddr addr = (RefAddr) addrs.nextElement();
			String name = addr.getType();
			String value = (String) addr.getContent();
			backendParameters.put(name,value);
		}
	}
	
	protected void initBeanParameters() {
		// Check out most generic parameters
		backendType     = backendParameters.get(BACKEND_TYPE);
		backendFile     = backendParameters.get(BACKEND_FILE);
		backendUrl      = backendParameters.get(BACKEND_URL);
		backendHost     = backendParameters.get(BACKEND_HOST);
		backendPort     = backendParameters.get(BACKEND_PORT);
		backendDatabase = backendParameters.get(BACKEND_DB);
		backendUsername = backendParameters.get(BACKEND_USER);
		backendPassword = backendParameters.get(BACKEND_PASS);
		backendClass    = backendParameters.get(BACKEND_CLASS);
		backendReadOnly = backendParameters.get(BACKEND_READ_ONLY);
		if (backendHost==null) {
			backendHost = "localhost";
		}
		if (backendHost.isEmpty()) {
			throw new IllegalArgumentException("Can't work with empty "+BACKEND_HOST+" attribute.");
		}
	}
	
	public URL checkBackendUrl() {
		if (backendUrl==null) {
			throw new IllegalArgumentException("Can't work with null "+BACKEND_URL+" attribute for: "+jndiObjectName);
		}
		if (backendUrl.isEmpty()) {
			throw new IllegalArgumentException("Can't work with empty "+BACKEND_URL+" attribute for: "+jndiObjectName);
		}
		try {
			return new URL(backendUrl);
		} catch (MalformedURLException e) {
			throw new IllegalArgumentException("Could not convert "+backendUrl+" to url.",e);
		}
	}
	
	public File checkFile() {
		if (backendFile==null) {
			throw new IllegalArgumentException("Can't work with null "+BACKEND_FILE+" attribute for: "+jndiObjectName);
		}
		if (backendFile.isEmpty()) {
			throw new IllegalArgumentException("Can't work with empty "+BACKEND_FILE+" attribute for: "+jndiObjectName);
		}
		File file = new File(backendFile);
		if (file.exists()==false) {
			throw new IllegalStateException("File not found: "+file.getName()+" for: "+jndiObjectName);
		}
		if (file.canRead()==false) {
			throw new IllegalStateException("Can't read file: "+file+" for: "+jndiObjectName);
		}
		return file;
	}
	
	/**
	 * @return the jndiObject
	 */
	public Object getJndiObject() {
		return jndiObject;
	}
	
	/**
	 * @return the jndiNamePath
	 */
	public Name getJndiNamePath() {
		return jndiNamePath;
	}
	
	/**
	 * @return the jndiContext
	 */
	public Context getJndiContext() {
		return jndiContext;
	}
	
	/**
	 * @return the jndiEnv
	 */
	public Hashtable<?, ?> getJndiEnv() {
		return jndiEnv;
	}
	
	/**
	 * @return the jndiObjectName
	 */
	public String getJndiObjectName() {
		return jndiObjectName;
	}
	
	/**
	 * @return the backendType
	 */
	public String getBackendType() {
		return backendType;
	}
	
	/**
	 * @return the backendFile
	 */
	public String getBackendFile() {
		return backendFile;
	}
	
	/**
	 * @return the backendUrl
	 */
	public String getBackendUrl() {
		return backendUrl;
	}
	
	/**
	 * @return the backendHost
	 */
	public String getBackendHost() {
		return backendHost;
	}
	
	/**
	 * @return the backendPort
	 */
	public String getBackendPort() {
		return backendPort;
	}
	
	/**
	 * @return the backendDatabase
	 */
	public String getBackendDatabase() {
		return backendDatabase;
	}
	
	/**
	 * @return the backendUsername
	 */
	public String getBackendUsername() {
		return backendUsername;
	}
	
	/**
	 * @return the backendPassword
	 */
	public String getBackendPassword() {
		return backendPassword;
	}
	
	/**
	 * @return the backendClass
	 */
	public String getBackendClass() {
		return backendClass;
	}
	
	/**
	 * @return the backendReadOnly
	 */
	public String getBackendReadOnly() {
		return backendReadOnly;
	}
	
	/**
	 * @return the backendParameters
	 */
	public Map<String, String> getBackendParameters() {
		return backendParameters;
	}
}
