/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.mongodb;

import java.util.ArrayList;
import java.util.EnumSet;
import java.util.List;
import java.util.logging.Logger;

import org.bson.types.ObjectId;

import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.list.DefaultVascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendList;
import net.forwardfire.vasc.backend.list.VascBackendListFeature;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;

import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.BasicDBObject;

/**
 * MongodbVascBackendList Provides backend for mongodb.
 * 
 * @author Willem Cazander
 * @version 1.0 Dec 30, 2011
 */
public class MongodbVascBackendList extends AbstractMongodbVascBackend implements VascBackendList<ObjectId,BasicDBObject> {

	private Logger logger = null;
	private final EnumSet<VascBackendListFeature> features = EnumSet.noneOf(VascBackendListFeature.class);
	private final List<String> rowActions = new ArrayList<String>();
	
	public MongodbVascBackendList() {
		logger = Logger.getLogger(MongodbVascBackendList.class.getName());
		features.add(VascBackendListFeature.PAGE_DATA);
		rowActions.add(ROW_ACTION_DELETE);
	}
	
	@Override
	public EnumSet<VascBackendListFeature> getSupportedFeatures() {
		return features;
	}
	
	private long fetchTotalExecuteSize(VascBackendListRequest state) {
		DBCollection coll = getDBCollection();
		DBObject query = createFilterQuery(state);
		long result = coll.count(query);
		return result;
	}

	private DBObject createFilterQuery(VascBackendListRequest state) {
		DBObject query = new BasicDBObject();
		for (String key:state.getDataParameterKeys()) {
			Object value = state.getDataParameter(key);
			query.put(key, value);
			logger.finer("Setting query parameter key: '"+key+"' value: '"+value+"'");
		}
		return query;
	}
	
	@Override
	public VascBackendListResult<BasicDBObject> execute(VascBackendListRequest state) throws VascBackendException {
		return new DefaultVascBackendListResult<BasicDBObject>(executeList(state),fetchTotalExecuteSize(state));
	}
	
	private List<BasicDBObject> executeList(VascBackendListRequest state) throws VascBackendException {
		DBCollection coll = getDBCollection();
		DBObject query = createFilterQuery(state);
		DBCursor cur = coll.find(query);
		if (state.getPageSize()>0) {
			cur.limit(state.getPageSize());
			cur.skip(state.getPageIndex());
		}
		List<BasicDBObject> result = new ArrayList<BasicDBObject>(cur.count());
		while (cur.hasNext()) {
			DBObject row = cur.next();
			result.add((BasicDBObject)row);
		}
		return result;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.VascBackend#createVascBackendAccessDataRecord()
	 */
	@Override
	public VascBackendAccessDataRecord<ObjectId, BasicDBObject> createVascBackendAccessDataRecord() {
		return new MongodbVascBackendAccessDataRecord();
	}
	
	@Override
	public List<String> getSupportedRowActions() {
		return rowActions;
	}
	
	@Override
	public void executeRowAction(VascBackendListRequest request,List<ObjectId> recordPKs, String actionName) throws VascBackendException {
		if (ROW_ACTION_DELETE.equals(actionName)) {
			for (ObjectId objectId:recordPKs) {
				deleteById(objectId);
			}
		}
	}
}
