/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.backend.proxy;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.entry.VascEntryBackendEventListener;
import net.forwardfire.vasc.core.entry.VascEntryBackendEventListener.VascBackendEventType;

/**
* Holds !! and fires the backend event listeners.
* 
* @author Willem Cazander
* @version 1.0 Sep 21, 2010
*/
public class VascBackendProxyEventExecutor<DATA_PK extends Serializable,DATA_RECORD extends Serializable> extends AbstractVascBackendProxy<DATA_PK,DATA_RECORD> {
	
	private List<VascEntryBackendEventListener> listeners = null;
	
	/**
	 * @see java.lang.Object#clone()
	 */
	@Override
	public VascBackendProxy<DATA_PK,DATA_RECORD> clone() throws CloneNotSupportedException {
		VascBackendProxyEventExecutor<DATA_PK,DATA_RECORD> result = new VascBackendProxyEventExecutor<>();
		return result;
	}
	
	@Override
	public void initProxy(VascBackendCrud<Serializable> backend,VascEntry entry) {
		super.initProxy(backend, entry);
		this.entry=entry;
		this.listeners=new ArrayList<VascEntryBackendEventListener>(10);
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if (cl==null) {
			cl = entry.getClass().getClassLoader();
		}
		for (String clazz:entry.getVascEntryBackendEventListeners()) {
			VascEntryBackendEventListener listener;
			try {
				listener = (VascEntryBackendEventListener)cl.loadClass(clazz).newInstance();
			} catch (Exception e) {
				throw new RuntimeException("Could not load VascEntryBackendEventListener of: "+clazz);
			}
			listeners.add(listener);
		}
	}
	
	public void addVascEntryBackendEventListener(VascEntryBackendEventListener listener) {
		listeners.add(listener);
	}
	
	public void removeVascEntryBackendEventListener(VascEntryBackendEventListener listener) {
		listeners.remove(listener);
	}
	
	public List<VascEntryBackendEventListener> getVascEntryBackendEventListeners() {
		return listeners;
	}
	
	private void fireVascEvent(VascBackendEventType type, Object data) {
		for (int i=0;i<listeners.size();i++) {
			VascEntryBackendEventListener l = listeners.get(i);
			VascBackendEventType[] types = l.getEventTypes();
			if (types==null) {
				continue;
			}
			if (types.length==0) {
				continue;
			}
			for (int t=0;t<types.length;t++) {
				VascBackendEventType typeListener = types[t];
				if (typeListener==type) {
					l.vascEvent(entry, data);
				}
			}
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#delete(java.lang.Object)
	 */
	@Override
	public void delete(Serializable object) throws VascBackendException {
		fireVascEvent(VascBackendEventType.PRE_DELETE,object);
		super.delete(object);
		fireVascEvent(VascBackendEventType.POST_DELETE,object);
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#doRecordMoveDownById(net.forwardfire.vasc.backend.list.VascBackendListRequest, java.lang.Object)
	 */
	@Override
	public long doRecordMoveDownById(VascBackendListRequest state, Object primaryId) throws VascBackendException {
		fireVascEvent(VascBackendEventType.PRE_MOVE_DOWN,primaryId);
		Long result = super.doRecordMoveDownById(state, primaryId);
		fireVascEvent(VascBackendEventType.POST_MOVE_DOWN,result);
		return result;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#doRecordMoveUpById(net.forwardfire.vasc.backend.list.VascBackendListRequest, java.lang.Object)
	 */
	@Override
	public long doRecordMoveUpById(VascBackendListRequest state, Object primaryId) throws VascBackendException {
		fireVascEvent(VascBackendEventType.PRE_MOVE_UP,primaryId);
		Long result = super.doRecordMoveUpById(state, primaryId);
		fireVascEvent(VascBackendEventType.POST_MOVE_UP,result);
		return result;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#execute(net.forwardfire.vasc.backend.list.VascBackendListRequest)
	 */
	@Override
	public VascBackendListResult<Serializable> execute(VascBackendListRequest state) throws VascBackendException {
		fireVascEvent(VascBackendEventType.PRE_EXECUTE,state);
		VascBackendListResult<Serializable> result = super.execute(state);
		fireVascEvent(VascBackendEventType.POST_EXECUTE,result);
		return result;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#merge(java.lang.Object)
	 */
	@Override
	public Serializable merge(Serializable object) throws VascBackendException {
		fireVascEvent(VascBackendEventType.PRE_MERGE,object);
		Serializable result = super.merge(object);
		fireVascEvent(VascBackendEventType.POST_MERGE,result);
		return result;
	}
	
	/**
	 * @see net.forwardfire.vasc.backend.proxy.AbstractVascBackendProxy#persist(java.lang.Object)
	 */
	@Override
	public void persist(Serializable object) throws VascBackendException {
		fireVascEvent(VascBackendEventType.PRE_PERSIST,object);
		super.persist(object);
		fireVascEvent(VascBackendEventType.POST_PERSIST,object);
	}
}