/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.core;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * AbstractVascEntryFieldLocal
 * 
 * @author Willem Cazander
 * @version 1.0 Jun 5, 2012
 */
abstract public class AbstractVascEntryControllerLocal implements VascEntryControllerLocal {

	private Map<String,VascEntryLocal> entries = null;
	private Map<String,VascEntryGroupLocal> groups = null;	
	
	public AbstractVascEntryControllerLocal() {
		entries = Collections.synchronizedMap(new HashMap<String,VascEntryLocal>(1000));
		groups = Collections.synchronizedMap(new HashMap<String,VascEntryGroupLocal>(100));
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryControllerLocal#addVascEntry(net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public void addVascEntry(VascEntryLocal entry) {
		if (entry==null) {
			throw new NullPointerException("Can't add null VascEntry.");
		}
		if (entry.getId()==null) {
			throw new NullPointerException("Can't add VascEntry with null Id.");
		}
		entries.put(entry.getId(), entry);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryControllerLocal#removeVascEntry(java.lang.String)
	 */
	@Override
	public void removeVascEntry(String entryId) {
		entries.remove(entryId);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryController#getVascEntryById(java.lang.String)
	 */
	@Override
	public VascEntry getVascEntryById(String id) {
		VascEntryLocal entry = entries.get(id);
		if (entry==null) {
			throw new NullPointerException("Could not find vasc entry with id: "+id);
		}
		try {
			return entry.clone();
		} catch (CloneNotSupportedException e) {
			throw new NullPointerException("Could not clone entry: "+e.getMessage());
		}
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryControllerLocal#getMasterVascEntryById(java.lang.String)
	 */
	@Override
	public VascEntryLocal getMasterVascEntryById(String entryId) {
		VascEntryLocal entry = entries.get(entryId);
		return entry;
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryController#getVascEntryIds()
	 */
	@Override
	public List<String> getVascEntryIds() {
		List<String> result = new ArrayList<String>(entries.keySet());
		Collections.sort(result); // lets do abc for consistance behauvior.
		return result;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryController#getVascEntryByGroupId(java.lang.String)
	 */
	@Override
	public List<String> getVascEntryByGroupId(String groupId) {
		if (groupId==null) {
			throw new NullPointerException("Can't search null groupId.");
		}
		List<String> result = new ArrayList<String>(40);
		for (VascEntryLocal entry:entries.values()) {
			if (groupId.equals(entry.getVascGroupId())) {
				result.add(entry.getId());
			}
		}
		return result;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryController#getVascEntryGroupById(java.lang.String)
	 */
	@Override
	public VascEntryGroup getVascEntryGroupById(String id) {
		VascEntryGroupLocal result = groups.get(id);
		if (result==null) {
			return null;
		}
		try {
			return result.clone();
		} catch (CloneNotSupportedException e) {
			throw new NullPointerException("Could not clone entry: "+e.getMessage());
		}
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryController#getVascEntryGroupIds()
	 */
	@Override
	public List<String> getVascEntryGroupIds() {
		List<String> result = new ArrayList<String>(groups.keySet());
		Collections.sort(result); // lets do abc for consistance behauvior.
		return result;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryControllerLocal#addVascEntryGroup(net.forwardfire.vasc.core.VascEntryGroupLocal)
	 */
	@Override
	public void addVascEntryGroup(VascEntryGroupLocal group) {
		if (group==null) {
			throw new NullPointerException("Can't add null VascEntryGroup.");
		}
		if (group.getId()==null) {
			throw new NullPointerException("Can't add VascEntryGroup with null Id.");
		}
		
		// TODO: move 
		if (group.getName()==null) {
			group.setName("vasc.entry.group."+group.getId()+".name");
		}
		if (group.getDescription()==null) {
			group.setDescription("vasc.entry.group."+group.getId()+".description");
		}
		if (group.getImage()==null) {
			group.setImage("vasc.entry.group."+group.getId()+".image");
		}
		//group.getHelpId()
		
		groups.put(group.getId(), group);
	}
}
