/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.ResourceBundle;

import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.backend.proxy.VascBackendProxy;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryConfigControllerLocal;
import net.forwardfire.vasc.core.VascEntryConfigFinalizer;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryFieldLocal;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.VascEntryState;
import net.forwardfire.vasc.core.VascInterfaceKey;
import net.forwardfire.vasc.core.VascInterfaceKeyFrontend;
import net.forwardfire.vasc.core.VascInterfaceLoader;
import net.forwardfire.vasc.core.entry.VascEntryExport;
import net.forwardfire.vasc.core.entry.VascEntryFieldValidatorService;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.VascFrontendControllerLocal;
import net.forwardfire.vasc.frontend.VascFrontendEntry;
import net.forwardfire.vasc.impl.entry.DefaultVascEntryResourceResolver;
import net.forwardfire.vasc.impl.frontend.DefaultVascFrontendHelper;

/**
 * DefaultVascEntryConfigController runs all VascEntryConfigFinalizers on VascEntry.
 * 
 * @author Willem Cazander
 * @version 1.0 May 9, 2012
 */
public class DefaultVascEntryConfigController implements VascEntryConfigControllerLocal {

	private List<VascEntryConfigFinalizer> configFinalizers = null;
	private List<VascBackendProxy> backendProxies = null;
	private Map<String,VascEntryExport> dataExporters = null;
	private List<VascEntryFieldValidatorService> fieldValidators = null;
	private Map<VascInterfaceKey,Class<?>> vascInterfaceImpls = null;
	private Map<VascInterfaceKeyFrontend,Class<?>> vascInterfaceImplFrontends = null;
	private VascInterfaceLoader vascInterfaceLoader = null;
	private VascBackendListRequest masterVascBackendState = null;
	private String resourceBundle = null;
	
	public DefaultVascEntryConfigController() {
		configFinalizers = new ArrayList<VascEntryConfigFinalizer>(10);
		backendProxies = new ArrayList<VascBackendProxy>(10);
		dataExporters = new HashMap<String,VascEntryExport>(10);
		fieldValidators = new ArrayList<VascEntryFieldValidatorService>(5);
		vascInterfaceImpls = new HashMap<VascInterfaceKey,Class<?>>(VascInterfaceKey.values().length);
		vascInterfaceImplFrontends = new HashMap<VascInterfaceKeyFrontend,Class<?>>(VascInterfaceKeyFrontend.values().length);
		vascInterfaceLoader = new DefaultVascInterfaceLoader(this);
		masterVascBackendState = new VascBackendListRequest();
		masterVascBackendState.setPageSize(100);
		masterVascBackendState.setPageSizeMax(1500);
	}

	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#getVascInterfaceLoader()
	 */
	@Override
	public VascInterfaceLoader getVascInterfaceLoader() {
		return vascInterfaceLoader;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#createVascInterfaceImplemention(net.forwardfire.vasc.core.VascInterfaceKey)
	 */
	@Override
	public Object createVascInterfaceImplemention(VascInterfaceKey key) {
		Class<?> classImpl = vascInterfaceImpls.get(key);
		if (classImpl==null) {
			throw new IllegalStateException("Could not find impl class for key: "+key);
		}
		try {
			Object objectImpl = classImpl.newInstance();
			return objectImpl;
		} catch (Exception e) {
			throw new RuntimeException("Could not instance of class: "+classImpl,e);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigControllerLocal#putVascInterfaceImplemention(net.forwardfire.vasc.core.VascInterfaceKey, java.lang.Class)
	 */
	@Override
	public void putVascInterfaceImplemention(VascInterfaceKey key,Class<?> interfaceImpl) {
		if (key==null) {
			throw new NullPointerException("Can't add null interface key.");
		}
		if (interfaceImpl==null) {
			throw new NullPointerException("Can't add null interface class.");
		}
		vascInterfaceImpls.put(key,interfaceImpl);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#createVascInterfaceImplementionFrontend(net.forwardfire.vasc.core.VascInterfaceKeyFrontend, net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public Object createVascInterfaceImplementionFrontend(VascInterfaceKeyFrontend key, VascEntry entry) {
		Class<?> classImpl = vascInterfaceImplFrontends.get(key);
		if (classImpl==null) {
			throw new IllegalStateException("Could not find impl class for key: "+key);
		}
		try {
			Object objectImpl = classImpl.newInstance();
			if (objectImpl instanceof VascFrontendEntry ) {
				((VascFrontendEntry)objectImpl).init(entry);
			}
			return objectImpl;
		} catch (Exception e) {
			throw new RuntimeException("Could not instance of class: "+classImpl,e);
		}
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigControllerLocal#putVascInterfaceImplementionFrontend(net.forwardfire.vasc.core.VascInterfaceKeyFrontend, java.lang.Class)
	 */
	@Override
	public void putVascInterfaceImplementionFrontend(VascInterfaceKeyFrontend key, Class<?> interfaceImpl) {
		if (key==null) {
			throw new NullPointerException("Can't add null interface key.");
		}
		if (interfaceImpl==null) {
			throw new NullPointerException("Can't add null interface class.");
		}
		vascInterfaceImplFrontends.put(key,interfaceImpl);
	}
	
	@Override
	public VascEntry configVascEntry(VascController vascController,String entryId) throws VascException {
		VascEntryLocal vascEntry = (VascEntryLocal)vascController.getVascEntryController().getVascEntryById(entryId);
		configVascEntry(vascController,vascEntry);
		return vascEntry;
	}
	
	@Override
	public void configVascEntry(VascController vascController,VascEntryLocal vascEntry) throws VascException {
		for (VascEntryConfigFinalizer finalizer:configFinalizers) {
			finalizer.configVascEntry(vascController,vascEntry);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#configVascBackendProxied(net.forwardfire.vasc.core.VascController, net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public VascBackendCrud<Serializable,Serializable> configVascBackendProxied(VascController vascController,VascEntry vascEntry) throws VascException {
		VascBackendCrud<Serializable,Serializable> realBackend = vascController.getVascBackendController().getVascBackendById(VascBackendCrud.class,vascEntry.getBackendId());
		if (realBackend==null) {
			throw new VascException("Could not find backend: "+vascEntry.getBackendId());
		}
		return configVascBackendProxied(vascController,vascEntry,realBackend);
	}
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#configVascBackendProxied(net.forwardfire.vasc.core.VascController, net.forwardfire.vasc.core.VascEntry, net.forwardfire.vasc.backend.crud.VascBackendCrud)
	 */
	@Override
	public VascBackendCrud<Serializable,Serializable> configVascBackendProxied(VascController vascController,VascEntry vascEntry, VascBackendCrud<Serializable,Serializable> realBackend) throws VascException {
		VascBackendCrud<Serializable,Serializable> backend = realBackend;
		for (VascBackendProxy proxy:backendProxies) {
			VascBackendProxy proxyClone;
			try {
				proxyClone = proxy.clone();
			} catch (CloneNotSupportedException e) {
				throw new VascException(e);
			}
			proxyClone.initProxy(backend, vascEntry);
			if (proxyClone.isProxyNeeded()==false) {
				continue;
			}
			backend = proxyClone;
		}
		return backend;
	}
	
	/**
	 * @throws VascException 
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#configVascFrontendController(net.forwardfire.vasc.core.VascController, net.forwardfire.vasc.core.VascEntry, java.util.Locale)
	 */
	@Override
	public void configVascFrontendController(VascController vascController,VascEntryLocal entry) throws VascException {

		// get the interface loader and create some impl objects
		VascInterfaceLoader loader					= getVascInterfaceLoader();
		VascFrontendControllerLocal controller		= loader.createVascFrontendControllerLocalImpl();
		VascBackendListRequest backendState				= loader.createVascBackendStateImpl();
		VascEntryState state						= loader.createVascEntryStateImpl();
		
		// Add impl to tree
		state.setVascBackendState(backendState);
		controller.setVascEntryState(state);
		controller.setVascController(vascController);
		entry.setVascFrontendController(controller);			// <------ This is the end result.
		
		// Add some generic frontend impl code.
		controller.setVascFrontendPageInfo(					loader.createVascFrontendPagerImpl(entry));
		controller.setVascFrontendActions(					loader.createVascFrontendActionsImpl(entry));
		controller.setVascFrontendDataSelector(				loader.createVascFrontendDataSelectorImpl(entry));
		controller.setVascFrontendUserController(			loader.createVascFrontendUserControllerImpl(entry));
		controller.setVascFrontendUserSettingController(	loader.createVascFrontendUserSettingControllerImpl(entry));
		
		// Config backend state from master template
		VascBackendListRequest master = getMasterVascBackendState();
		for (String key:master.getDataParameterKeys()) {
			backendState.setDataParameter(key, master.getDataParameter(key));
		}
		backendState.setPageSize(master.getPageSize());
		backendState.setPageSizeMax(master.getPageSizeMax());
		
		// init resource loaders 
		if (getResourceBundle()==null) {
			controller.setVascEntryResourceResolver(new DefaultVascEntryResourceResolver());
		} else {
			Locale locale = controller.getVascFrontendUserController().getUserLocale();
			controller.setVascEntryResourceResolver(new DefaultVascEntryResourceResolver(ResourceBundle.getBundle(getResourceBundle(), locale)));
		}
		controller.setVascFrontendHelper(new DefaultVascFrontendHelper());
		//vascFrontendData.setVascEntryResourceImageResolver(new ImageResources());
		
		// Add validators
		for(VascEntryFieldValidatorService validator:getVascEntryFieldValidatorServices()) {
			controller.addVascValidatorService(validator);
		}
		
		// Add backend to entry
		VascBackendCrud<Serializable,Serializable> backend = vascController.getVascEntryConfigController().configVascBackendProxied(vascController, entry);
		controller.getVascEntryState().setVascBackend(backend);
		controller.getVascEntryState().setVascEntry(entry);
		
		// Fetch all data field value impl for this backend.
		for (VascEntryField field:entry.getVascEntryFields()) {
			if (field.getVascEntryFieldValue()==null) {
				VascBackendAccessDataRecord<Serializable,Serializable> v = backend.provideVascEntryFieldValue();
				((VascEntryFieldLocal)field).setVascEntryFieldValue(v);
			}
		}
	}

/* TODO check where this is left over

			for (String key:entry.getEntryParameterKeys()) {
				Object value = entry.getEntryParameter(key);
				if (value instanceof String==false) {
					continue;
				}
				String paraValue = (String)value;
				if (paraValue.startsWith("userPara")==false) {
					continue;
				}
				String[] ps = paraValue.split(":");
				String type = ps[1];
				String paraTypeValue = ps[2];
				
				SetParameterBackendListener listener = new SetParameterBackendListener(vui.userId,vui.username);
				listener.setName(key);
				listener.setType(type);
				listener.setValue(paraTypeValue);
				
				entry.getVa
			}


 */
	
	
	
	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#addVascEntryConfigFinalizer(net.forwardfire.vasc.core.VascEntryConfigFinalizer)
	 */
	@Override
	public void addVascEntryConfigFinalizer(VascEntryConfigFinalizer vascEntryConfigFinalizer) {
		configFinalizers.add(vascEntryConfigFinalizer);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#removeVascEntryConfigFinalizer(net.forwardfire.vasc.core.VascEntryConfigFinalizer)
	 */
	@Override
	public void removeVascEntryConfigFinalizer(VascEntryConfigFinalizer vascEntryConfigFinalizer) {
		configFinalizers.remove(vascEntryConfigFinalizer);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#getVascEntryConfigFinalizers()
	 */
	@Override
	public List<VascEntryConfigFinalizer> getVascEntryConfigFinalizers() {
		return configFinalizers;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#addVascBackendProxy(net.forwardfire.vasc.backend.proxy.VascBackendProxy)
	 */
	@Override
	public void addVascBackendProxy(VascBackendProxy proxy) {
		backendProxies.add(proxy);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#removeVascBackendProxy(net.forwardfire.vasc.backend.proxy.VascBackendProxy)
	 */
	@Override
	public void removeVascBackendProxy(VascBackendProxy proxy) {
		backendProxies.remove(proxy);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#getVascBackendProxies()
	 */
	@Override
	public List<VascBackendProxy> getVascBackendProxies() {
		return backendProxies;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#getVascEntryExporterById(java.lang.String)
	 */
	@Override
	public VascEntryExport getVascEntryExporterById(String exporterId) {
		return dataExporters.get(exporterId);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#addVascEntryExporter(net.forwardfire.vasc.core.entry.VascEntryExport)
	 */
	@Override
	public void addVascEntryExporter(VascEntryExport exporter) {
		dataExporters.put(exporter.getId(),exporter);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#removeVascEntryExporter(net.forwardfire.vasc.core.entry.VascEntryExport)
	 */
	@Override
	public void removeVascEntryExporter(VascEntryExport exporter) {
		dataExporters.remove(exporter.getId());
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#getVascEntryExporterIds()
	 */
	@Override
	public List<String> getVascEntryExporterIds() {
		return new ArrayList<String>(dataExporters.keySet());
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#getResourceBundle()
	 */
	@Override
	public String getResourceBundle() {
		return resourceBundle;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#setResourceBundle(java.lang.String)
	 */
	@Override
	public void setResourceBundle(String resourceBundle) {
		this.resourceBundle=resourceBundle;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#addVascEntryFieldValidatorService(net.forwardfire.vasc.core.entry.VascEntryFieldValidatorService)
	 */
	@Override
	public void addVascEntryFieldValidatorService(VascEntryFieldValidatorService validator) {
		fieldValidators.add(validator);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#removeVascEntryFieldValidatorService(net.forwardfire.vasc.core.entry.VascEntryFieldValidatorService)
	 */
	@Override
	public void removeVascEntryFieldValidatorService(VascEntryFieldValidatorService validator) {
		fieldValidators.remove(validator);
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigController#getVascEntryFieldValidatorServices()
	 */
	@Override
	public List<VascEntryFieldValidatorService> getVascEntryFieldValidatorServices() {
		return fieldValidators;
	}

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigControllerLocal#getMasterVascBackendState()
	 */
	@Override
	public VascBackendListRequest getMasterVascBackendState() {
		return masterVascBackendState;
	}
}