/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.entry;

import java.io.Serializable;
import java.lang.annotation.Annotation;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.lang.reflect.Modifier;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.logging.Logger;

import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.entry.VascEntryFieldValidatorService;
import net.forwardfire.vasc.core.VascException;

import org.hibernate.validator.ClassValidator;
import org.hibernate.validator.InvalidValue;
import org.hibernate.validator.MessageInterpolator;
import org.hibernate.validator.Validator;


/**
 * Executes the hibernate field validators
 * 
 * @author Willem Cazander
 * @version 1.0 May 13, 2009
 */
public class HibernateValidatorService implements VascEntryFieldValidatorService {

	private Logger logger = Logger.getLogger(HibernateValidatorService.class.getName());
	
	@SuppressWarnings("unchecked")
	@Override
	public List<String> validateObjectField(VascEntryField field, Object selectedRecord,Object objectValue) throws VascException {
		List<String> error = new ArrayList<String>(3);
		try {
			ClassValidator val = new ClassValidator(selectedRecord.getClass(),new VascHibernateMessage(field.getVascEntry()));
			InvalidValue[] ival = val.getInvalidValues(selectedRecord);
			logger.fine("Hiber Validating: "+ival.length);
			String prop = field.getBackendName();
			InvalidValue iv = findInvalidValueByProperty(ival,prop);
			
			if(iv!=null) {
				error.add(iv.getMessage());
			}
		} catch (Exception e) {
			throw new VascException(e);
		}
		return error;
	}
	
	private InvalidValue findInvalidValueByProperty(InvalidValue[] ival,String property) {
		for(InvalidValue iv:ival) {
			if(iv.getPropertyName().equals(property)) {
				return iv;
			}
		}
		return null;
	}
	
	class VascHibernateMessage implements MessageInterpolator, Serializable {
		
		private static final long serialVersionUID = -8241727232507976072L;
		//private static final Logger log = Logger.getLogger(VascHibernateMessage.class.getName());
		private Map<String, Object> annotationParameters = new HashMap<String, Object>();
		private String annotationMessage;
		private String interpolateMessage;
		private VascEntry vascEntry;
		
		public VascHibernateMessage(VascEntry vascEntry) {
			this.vascEntry=vascEntry;
		}
		
		public void initialize(Annotation annotation, MessageInterpolator defaultInterpolator) {
			Class<?> clazz = annotation.getClass();
			for ( Method method  : clazz.getMethods() ) {
				try {
					//FIXME remove non serilalization elements on writeObject?
					if ( method.getReturnType() != void.class
							&& method.getParameterTypes().length == 0
							&& ! Modifier.isStatic( method.getModifiers() ) ) {
							//cannot use an exclude list because the parameter name could match a method name
							annotationParameters.put( method.getName(), method.invoke( annotation ) );
					}
				} catch (IllegalAccessException e) {
					//really should not happen, but we degrade nicely
					//log.warning( "Unable to access {}", StringHelper.qualify( clazz.toString(), method.getName() ) );
				} catch (InvocationTargetException e) {
					//really should not happen, but we degrade nicely
					//log.warn( "Unable to access {}", StringHelper.qualify( clazz.toString(), method.getName() ) );
				}
			}
			annotationMessage = (String) annotationParameters.get( "message" );
			if (annotationMessage == null) {
				throw new IllegalArgumentException( "Annotation " + clazz + " does not have an (accessible) message attribute");
			}
			//do not resolve the property eagerly to allow validator.apply to work wo interpolator
		}
		
		@SuppressWarnings("rawtypes")
		@Override
		public String interpolate(String message, Validator validator, MessageInterpolator defaultInterpolator) {
			if ( annotationMessage!=null && annotationMessage.equals( message ) ) {
				//short cut
				if (interpolateMessage == null) {
					interpolateMessage = replace( annotationMessage );
				}
				return interpolateMessage;
			} else {
				//TODO keep them in a weak hash map, but this might not even be useful
				return replace( message );
			}
		}
		
		public String getAnnotationMessage() {
			return annotationMessage;
		}
		
		private String replace(String message) {
			StringTokenizer tokens = new StringTokenizer( message, "#{}", true );
			StringBuilder buf = new StringBuilder( 30 );
			boolean escaped = false;
			boolean el = false;
			while ( tokens.hasMoreTokens() ) {
				String token = tokens.nextToken();
				if ( !escaped && "#".equals( token ) ) {
					el = true;
				}
				if ( !el && "{".equals( token ) ) {
					escaped = true;
				} else if ( escaped && "}".equals( token ) ) {
					escaped = false;
				} else if ( !escaped ) {
					if ( "{".equals( token ) ) el = false;
					buf.append( token );
				} else {
					Object variable = annotationParameters.get( token );
					if ( variable != null ) {
						buf.append( variable );
					} else {
						String string = vascEntry.getVascFrontendController().getVascEntryResourceResolver().getTextValue(token);
						if ( string != null ) { 
							buf.append(replace(string));
						}
					}
				}
			}
			return buf.toString();
		}
	}
}
