/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.entry.config;

import java.io.Serializable;

import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryFieldSet;
import net.forwardfire.vasc.core.VascEntryConfigFinalizer;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.VascEntryLink;
import net.forwardfire.vasc.core.actions.VascAction;
import net.forwardfire.vasc.core.VascException;

/**
 * VascIdCheckFinalizer Checks the all objects have an ID.
 * 
 * @author Willem Cazander
 * @version 1.0 May 9, 2012
 */
public class VascIdCheckFinalizer implements VascEntryConfigFinalizer  {

	/**
	 * @see net.forwardfire.vasc.core.VascEntryConfigFinalizer#configVascEntry(net.forwardfire.vasc.core.VascController, net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public void configVascEntry(VascController vascController, VascEntryLocal entry)	throws VascException {

		// First Check if we all have ids
		String id = entry.getId();
		if (id==null) {
			throw new IllegalArgumentException("The VascEntry need an id.");
		}
		if (entry.getBackendId()==null) {
			throw new IllegalArgumentException("The VascEntry need an backendId in entryId: "+id);
		}
		for (VascEntryField vef:entry.getVascEntryFields()) {
			if (vef.getId()==null) {
				throw new IllegalArgumentException("All VascEntryField need an id in entryId: "+id);	
			}
		}
		for (VascEntryField vef:entry.getVascEntryListOptions()) {
			if (vef.getId()==null) {
				throw new IllegalArgumentException("All listOption VascEntryField to have an id in entryId: "+id);	
			}
		}
		
		// Check if backendId is valid
		VascBackendCrud<Serializable,Serializable> back = vascController.getVascBackendController().getVascBackendById(VascBackendCrud.class, entry.getBackendId() );
		if (back==null) {
			throw new IllegalArgumentException("The VascEntry backend is not found in backends: '"+entry.getBackendId()+"' in entryId: "+id); 
		}
		
		// optional field sets
		for (VascEntryFieldSet s:entry.getVascEntryFieldSets()) {
			String sid = s.getId();
			if (sid==null) {
				throw new IllegalArgumentException("All VascEntryFieldSet need an id in entryId: "+id);	
			}
			
			// check if refenced ids are avalible
			for (String fid:s.getVascEntryFieldIds()) {
				if (entry.getVascEntryFieldById(fid)==null) {
					throw new IllegalArgumentException("VascEntryFieldSet "+sid+" has non excisting field id: "+fid+" in entryId: "+id);
				}
			}
		}
		
		// Check if link entries excists
		for (VascEntryLink vle:entry.getVascEntryLinks()) {
			String vid = vle.getId();
			if (vid==null) {
				throw new IllegalArgumentException("All VascLinkEntry need an id in entryId: "+id);	
			}
			if (vle.getVascEntryId()==null) {
				throw new IllegalArgumentException("All VascLinkEntry need an vascEntryId: "+id);
			}
		}

		for (VascAction action:entry.getGlobalActions()) {
			String aid = action.getId();
			if (aid==null) {
				throw new IllegalArgumentException("Action has no id: "+action+" in entryId: "+id);
			}
		}
		for (VascAction action:entry.getExportActions()) {
			String aid = action.getId();
			if (aid==null) {
				throw new IllegalArgumentException("Action has no id: "+action+" in entryId: "+id);
			}
		}
		for (VascAction action:entry.getRowActions()) {
			String aid = action.getId();
			if (aid==null) {
				throw new IllegalArgumentException("Action has no id: "+action+" in entryId: "+id);
			}
		}
		for (VascAction action:entry.getColumnActions()) {
			String aid = action.getId();
			if (aid==null) {
				throw new IllegalArgumentException("Action has no id: "+action+" in entryId: "+id);
			}
		}
	}
}
