/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.frontend;

import java.io.Serializable;
import java.util.Date;
import java.util.logging.Logger;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.list.VascBackendListResult;
import net.forwardfire.vasc.backend.list.VascBackendListRequest;
import net.forwardfire.vasc.backend.proxy.VascProxyFilter;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.entry.VascEntryFrontendEventListener;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.VascFrontendActions;
import net.forwardfire.vasc.frontend.VascFrontendException;

/**
 * Default impl of default frontend actions
 * 
 * @author Willem Cazander
 * @version 1.0 Jan 22, 2012
 */
public class DefaultVascFrontendActions implements VascFrontendActions  {

	private Logger logger = Logger.getLogger(DefaultVascFrontendActions.class.getName());
	private VascEntry entry = null;

	@Override
	public void init(VascEntry entry) {
		this.entry=entry;
	}

	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendHelper#initEditObject(net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public Serializable createObject() throws VascFrontendException {
		entry.getVascFrontendController().fireVascFrontendEvent(entry,VascEntryFrontendEventListener.VascFrontendEventType.PRE_CREATE, null);
		Serializable object;
		try {
			object = entry.getVascFrontendController().getVascEntryState().getVascBackend().newRecord();
		} catch (VascBackendException e) {
			throw new VascFrontendException(e);
		} 
		if (object==null) {
			throw new IllegalStateException("Can't work with null object for backend storage.");
		}
		for (VascEntryField field:entry.getVascEntryFields()) {
			if (field.getDefaultValue()==null) {
				continue; // no default value to set.
			}
			try {
				Serializable value = field.getVascEntryFieldValue().getValue(field.getBackendName(), object);
				if (value!=null) {
					continue; // value is already set by backend creator.
				}
				Serializable defaultValue = field.getDefaultValue();
				if (defaultValue instanceof String) {
					String def = (String)defaultValue;
					if (def.equals("now()")) { // TODO: add default string parsers
						defaultValue = new Date();
					}
				}
				logger.finer("Setting default value for: "+field.getName()+" def: "+defaultValue);
				field.getVascEntryFieldValue().setValue(field.getBackendName(), object, defaultValue);
			} catch (VascBackendException ve) {
				throw new VascFrontendException(ve);
			}
		}
		entry.getVascFrontendController().fireVascFrontendEvent(entry,VascEntryFrontendEventListener.VascFrontendEventType.POST_CREATE, object);
		return object;
	}

	protected int removeObjectFromDataList(Serializable object) throws VascBackendException {
		int indexOld = entry.getVascFrontendController().getVascEntryState().getEntryDataList().indexOf(object);
		if (entry.getVascFrontendController().getVascEntryState().getEntryDataList().remove(object)) {
			return indexOld; // java worked well for use
		}
		
		// remove only work on (jpa)beans with an overrided equals method.
		// we lets do the search ourselfs here because we should know the primary key value
		VascEntryField field = entry.getVascEntryFieldById(entry.getPrimaryKeyFieldId());
		Object idObject = field.getVascEntryFieldValue().getValue(field.getBackendName(), object);
			
		// is only null when creating objects
		if (idObject!=null) {
			int index = 0;
			for (Serializable o:entry.getVascFrontendController().getVascEntryState().getEntryDataList()) {
				field = entry.getVascEntryFieldById(entry.getPrimaryKeyFieldId());
				Object id = field.getVascEntryFieldValue().getValue(field.getBackendName(), o);
				if (idObject.equals(id)) {
					break;
				}
				index++;
			}
			if (index<entry.getVascFrontendController().getVascEntryState().getEntryDataList().size()) {
				entry.getVascFrontendController().getVascEntryState().getEntryDataList().remove(index);
				return index;
			}
		}
		return 0; // make better (0=top of list)
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendActions#persistObject()
	 */
	@Override
	public void persistObject() throws VascFrontendException {
		try {
			saveObject(true);
		} catch (VascException e) {
			throw new VascFrontendException(e);
		}
	}

	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendActions#mergeObject()
	 */
	@Override
	public Serializable mergeObject() throws VascFrontendException {
		try {
			return saveObject(false);
		} catch (VascException e) {
			throw new VascFrontendException(e);
		}
	}

	protected Serializable saveObject(boolean persist) throws VascException {
		try {
			Serializable object = entry.getVascFrontendController().getVascEntryState().getEntryDataObject();
			Serializable result = null;
	
			entry.getVascFrontendController().fireVascFrontendEvent(entry,VascEntryFrontendEventListener.VascFrontendEventType.PRE_UPDATE,object);
			int index = removeObjectFromDataList(object);
			
			// save object on backend
			if (persist) {
				entry.getVascFrontendController().getVascEntryState().getVascBackend().persist(object);
				result = object; // TODO: fix persist
				entry.getVascFrontendController().getVascEntryState().setTotalBackendRecords(1+entry.getVascFrontendController().getVascEntryState().getTotalBackendRecords());
			} else {
				result = entry.getVascFrontendController().getVascEntryState().getVascBackend().merge(object);
			}
			
			// put object thrue the filters
			for (VascProxyFilter filter:entry.getVascBackendFilters()) {
				result = filter.filterObject(result);
			}
			
			// put object back in list
			entry.getVascFrontendController().getVascEntryState().getEntryDataList().add(index, result);
			entry.getVascFrontendController().getVascEntryState().setEntryDataObject(null);
			entry.getVascFrontendController().fireVascFrontendEvent(entry,VascEntryFrontendEventListener.VascFrontendEventType.POST_UPDATE,result);
			return result;
		} catch (VascBackendException e) {
			throw new VascException(e);
		}
	}

	/**
	 * Deletes the selected row object from the back-end and list and fires events.
	 * @param entry
	 * @param object
	 */
	@Override
	public void deleteObject() throws VascFrontendException {
		try {
			Serializable object = entry.getVascFrontendController().getVascEntryState().getEntryDataObject();
			entry.getVascFrontendController().fireVascFrontendEvent(entry,VascEntryFrontendEventListener.VascFrontendEventType.PRE_DELETE, object);
			entry.getVascFrontendController().getVascEntryState().getVascBackend().delete(object);
	
			removeObjectFromDataList(object);
			entry.getVascFrontendController().getVascEntryState().setEntryDataObject(null);
			entry.getVascFrontendController().getVascEntryState().setTotalBackendRecords(entry.getVascFrontendController().getVascEntryState().getTotalBackendRecords()-1);
			entry.getVascFrontendController().fireVascFrontendEvent(entry,VascEntryFrontendEventListener.VascFrontendEventType.POST_DELETE, object);
		} catch (VascBackendException ve) {
			throw new VascFrontendException(ve);
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendHelper#refreshData(net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public void refreshData() throws VascFrontendException {
		
		// Fire pre read event
		entry.getVascFrontendController().fireVascFrontendEvent(entry,VascEntryFrontendEventListener.VascFrontendEventType.PRE_READ, null);
		entry.getVascFrontendController().getVascEntryState().setEntryDataObject(null);
		VascBackendListRequest backendState = entry.getVascFrontendController().getVascEntryState().getVascBackendState();
		
		// check and correct max page size
		if (backendState.getPageSize() > backendState.getPageSizeMax()) {
			backendState.setPageSize(backendState.getPageSizeMax());
		}
		
		// Clear and copy parameters to backend state for query.
		backendState.removeDataParameterAll();
		for (String key:entry.getEntryParameterKeys()) {
			Object value = entry.getEntryParameter(key);
			if (value!=null) {
				backendState.setDataParameter(key, value);
			}
		}
		
		VascBackendListResult<Serializable> result;
		try {
			result = entry.getVascFrontendController().getVascEntryState().getVascBackend().execute(backendState);
		} catch (VascBackendException ve) {
			throw new VascFrontendException(ve);
		}
		
		// Set data in frontend
		entry.getVascFrontendController().getVascEntryState().setEntryDataList(result.getData());
		entry.getVascFrontendController().getVascEntryState().setTotalBackendRecords(result.getTotalRecordCount());
		
		// check if we need to change the current page if table size if changed in between
		int pages = new Long(result.getTotalRecordCount()/backendState.getPageSize()).intValue();
		if (backendState.getPageIndex() > pages) {
			backendState.setPageIndex(pages);
		}
		
		// Fire event post read
		entry.getVascFrontendController().fireVascFrontendEvent(entry,VascEntryFrontendEventListener.VascFrontendEventType.POST_READ, null);
	}

	@Override
	public void sortAction(VascEntryField field) throws VascFrontendException {
		String curSort = entry.getVascFrontendController().getVascEntryState().getVascBackendState().getSortField();
		if (field.getBackendName().equals(curSort)) {
			entry.getVascFrontendController().getVascEntryState().getVascBackendState().setSortAscending(!entry.getVascFrontendController().getVascEntryState().getVascBackendState().isSortAscending());
		}
		String sortID = field.getBackendName();
		entry.getVascFrontendController().getVascEntryState().getVascBackendState().setSortField(sortID);
		entry.getVascFrontendController().getVascEntryState().getVascBackendState().setPageIndex(0);
		entry.getVascFrontendController().fireVascFrontendEvent(entry,VascEntryFrontendEventListener.VascFrontendEventType.SORT, field);
		
		refreshData();
		entry.getVascFrontendController().getVascFrontend().renderView();
	}
	
	@Override
	public void searchAction(String searchString) throws VascFrontendException {
		entry.getVascFrontendController().getVascEntryState().getVascBackendState().setSearchString(searchString);
		entry.getVascFrontendController().getVascEntryState().getVascBackendState().setPageIndex(0);
		entry.getVascFrontendController().fireVascFrontendEvent(entry,VascEntryFrontendEventListener.VascFrontendEventType.SEARCH, searchString);		
		
		refreshData();
		entry.getVascFrontendController().getVascFrontend().renderView();
	}
	
	@Override
	public void pageAction(Integer pageIndex) throws VascFrontendException {
		if (pageIndex<1) {
			pageIndex = 0;
		}
		Long total = entry.getVascFrontendController().getVascEntryState().getTotalBackendRecords(); // note: total is only null when pageAction is done before first refresh,  which should never happen anyway.
		if (total!=null &&  pageIndex>(total/entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageSize())) {
			pageIndex = new Long(total/entry.getVascFrontendController().getVascEntryState().getVascBackendState().getPageSize()).intValue();
		}
		
		entry.getVascFrontendController().getVascEntryState().getVascBackendState().setPageIndex(pageIndex);
		entry.getVascFrontendController().fireVascFrontendEvent(entry,VascEntryFrontendEventListener.VascFrontendEventType.PAGE, pageIndex);
		
		// lets load data;
		refreshData();

		entry.getVascFrontendController().getVascFrontend().renderView();
	}
	
	@Override
	public void moveUpAction(Serializable record) throws VascFrontendException {
		if (entry.getVascFrontendController().getVascEntryState().getVascBackend().isRecordMoveable()) {
			try {
				VascEntryField p = entry.getVascEntryFieldById(entry.getPrimaryKeyFieldId());
				Object primaryId = p.getVascEntryFieldValue().getValue(p.getBackendName(), record); 
				entry.getVascFrontendController().getVascEntryState().getVascBackend().doRecordMoveUpById(entry.getVascFrontendController().getVascEntryState().getVascBackendState(),primaryId);
			} catch (VascBackendException ve) {
				throw new VascFrontendException(ve);
			}

			
			// lets load data;
			refreshData();
		}
		entry.getVascFrontendController().getVascFrontend().renderView();
	}
	
	@Override
	public void moveDownAction(Serializable record) throws VascFrontendException {
		if (entry.getVascFrontendController().getVascEntryState().getVascBackend().isRecordMoveable()) {
			
			try {
				VascEntryField p = entry.getVascEntryFieldById(entry.getPrimaryKeyFieldId());
				Object primaryId = p.getVascEntryFieldValue().getValue(p.getBackendName(), record); 
				entry.getVascFrontendController().getVascEntryState().getVascBackend().doRecordMoveDownById(entry.getVascFrontendController().getVascEntryState().getVascBackendState(),primaryId);
			} catch (VascBackendException ve) {
				throw new VascFrontendException(ve);
			}
			
			// lets load data;
			refreshData();
		}
		entry.getVascFrontendController().getVascFrontend().renderView();
	}
}