/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.frontend;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.backend.VascBackendAccessDataRecord;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryField;
import net.forwardfire.vasc.core.VascEntryLink;
import net.forwardfire.vasc.core.VascEntryLinkType;
import net.forwardfire.vasc.core.actions.RowVascAction;
import net.forwardfire.vasc.core.entry.VascEntryFieldValidatorService;
import net.forwardfire.vasc.core.ui.VascUIComponent;
import net.forwardfire.vasc.core.VascException;
import net.forwardfire.vasc.frontend.VascFrontendHelper;

/**
 * 
 * @author Willem Cazander
 * @version 1.0 Apr 28, 2007
 */
public class DefaultVascFrontendHelper implements VascFrontendHelper  {

	private Logger logger = Logger.getLogger(DefaultVascFrontendHelper.class.getName());
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendHelper#getTotalColumnsWidth(net.forwardfire.vasc.core.VascEntry)
	 */
	@Override
	public Integer getTotalColumnsWidth(VascEntry entry) {
    	int result = 0;
    	for(VascEntryField c:entry.getVascEntryFields()) {
    		if(c.getSizeList()==null) {
    			logger.finer("Column no size: "+c.getName());
    		} else {
    			result+=c.getSizeList();
    		}
    	}
    	return result;
	}
	
	@Override
	public List<VascEntryLink> getVascLinkEntryByType(VascEntry entry,VascEntryLinkType type) {
		List<VascEntryLink> result = new ArrayList<VascEntryLink>(10);
		for (VascEntryLink link:entry.getVascEntryLinks()) {
			if (type==null) {
				result.add(link);
				continue;
			}
			if (type.equals(link.getVascEntryLinkType())) {
				result.add(link);
			}
		}
		return result;
	}
	
	@Override
	public void headerOptionsCreatedFillData(VascEntry entry) throws VascException {
		
		// fix conv defs of options to object. ?
		
		entry.getVascFrontendController().getVascFrontendActions().refreshData();
	}
	
	/**
	 * @see net.forwardfire.vasc.frontend.VascFrontendHelper#validateObjectField(net.forwardfire.vasc.core.VascEntryField, java.lang.Object)
	 */
	@Override
	public List<String> validateObjectField(VascEntryField field) throws VascException {
		if (field==null) {
			throw new NullPointerException("Can't validate null field.");
		}
		VascEntry entry = field.getVascEntry();
		if (entry.getVascFrontendController().getVascEntryState().getEntryDataObject()==null) {
			throw new NullPointerException("Can't validate null entry object.");
		}
		List<String> error = new ArrayList<String>(3);
		
		// skip non-create and non-edit fields
		if (entry.getVascFrontendController().getVascFrontendDataSelector().isFieldCreate(field) == false &
				entry.getVascFrontendController().getVascEntryState().isEditCreate()) {
			return error;
		}
		if (entry.getVascFrontendController().getVascFrontendDataSelector().isFieldEditReadOnly(field) &
				entry.getVascFrontendController().getVascEntryState().isEditCreate()==false) {
			return error;
		}
		
		Serializable objectSelected = entry.getVascFrontendController().getVascEntryState().getEntryDataObject();
		try {
			Object objectValue = field.getVascEntryFieldValue().getValue(field.getBackendName(), objectSelected);
			for (VascEntryFieldValidatorService s:entry.getVascFrontendController().getVascValidatorServices()) {
				error.addAll(s.validateObjectField(field, objectSelected, objectValue));
			}
			return error;
		} catch (VascBackendException e) {
			throw new VascException(e);
		}
	}

	/**
	 * 
	 */
	@Override
	public boolean validateAndSetErrorText(VascEntry entry) throws VascException {		
		boolean hadError = false;
		for (VascEntryField field:entry.getVascEntryFields()) {
			VascUIComponent comp = entry.getVascFrontendController().getFieldVascUIComponent(field);
			List<String> error = validateObjectField(field);
			logger.info("Check field: "+field.getId()+" comp: "+comp+" Errors: "+error.size());
			if  (error.isEmpty()) {
				if (comp!=null) {
					comp.setErrorText(null);
				}
				continue;
			}
			if (comp==null) {
				logger.warning("Field: "+field.getId()+" gives errors but no UI component to display.");
				continue;
			}
			hadError=true;
			StringBuffer buf = new StringBuffer(100);
			for (String s:error) {
				buf.append(s);
				buf.append('\n');
			}
			comp.setErrorText(buf.toString());
		}
		return hadError;
	}
	
	@Override
	public void editReadOnlyUIComponents(VascEntry entry) {
		// reset edit read only 
		for (VascEntryField field:entry.getVascEntryFields()) {
			if (entry.getVascFrontendController().getFieldVascUIComponent(field)==null) {
				continue;
			}
			
			// TODO: move back to rendered when jsf fixes
			if (entry.getVascFrontendController().getVascFrontendDataSelector().isFieldCreate(field) == false &
					entry.getVascFrontendController().getVascEntryState().isEditCreate()) {
				//entry.getVascFrontendData().getFieldVascUIComponent(f).setRendered(false);
				entry.getVascFrontendController().getFieldVascUIComponent(field).setDisabled(true);
			} else {
				//entry.getVascFrontendData().getFieldVascUIComponent(f).setRendered(true);
				entry.getVascFrontendController().getFieldVascUIComponent(field).setDisabled(false);
			}
			
			// only  when editing set edit readonlys
			if (entry.getVascFrontendController().getVascFrontendDataSelector().isFieldEditReadOnly(field) &
					entry.getVascFrontendController().getVascEntryState().isEditCreate()==false) {
				entry.getVascFrontendController().getFieldVascUIComponent(field).setDisabled(true);
			} else {
				if (entry.getVascFrontendController().getVascEntryState().isEditCreate()==false) { // todo: remove when jsf fixes
					entry.getVascFrontendController().getFieldVascUIComponent(field).setDisabled(false);
				}
			}
		}
	}
	
	@Override
	public List<RowVascAction> getMultiRowActions(VascEntry entry) {
		List<RowVascAction> result = new ArrayList<RowVascAction>(5);
		for (RowVascAction a:entry.getRowActions()) {
			if (a.isMultiRowAction()) {
				result.add(a);
			}
		}
		return result;
	}
	
	@Override
	public String getSelectedDisplayName(VascEntry entry) {
		Serializable row = entry.getVascFrontendController().getVascEntryState().getEntryDataObject();
		if (row==null) {
			return "no-selection";
		}		
		VascEntryField v = entry.getVascEntryFieldById(entry.getDisplayNameFieldId());
		VascBackendAccessDataRecord<Serializable,Serializable> ve = v.getVascEntryFieldValue();
		String result = "no-data";
		try {
			result = ""+ve.getValue(v.getBackendName(), row);
		} catch (VascBackendException e) {
			throw new RuntimeException("Could not get selected name DisplayValue: "+e.getMessage(),e);
		}
		return result;
	}
	
	@Override
	public String getParentSelectedDisplayName(VascEntry entry) {
		if (entry.getVascFrontendController().getVascEntryState().getParent()==null) {
			return ""; // no parent
		}
		
		VascEntry parent = entry.getVascFrontendController().getVascEntryState().getParent().getVascEntry();
		Serializable row = entry.getVascFrontendController().getVascEntryState().getParent().getEntryDataObject();
		if (row==null) {
			return "no-selection";
		}		
		VascEntryField v = parent.getVascEntryFieldById(parent.getDisplayNameFieldId());
		VascBackendAccessDataRecord<Serializable,Serializable> ve = v.getVascEntryFieldValue();
		String result = "no-data";
		try {
			result = ""+ve.getValue(v.getBackendName(), row);
		} catch (VascBackendException e) {
			throw new RuntimeException("Could not get parent name DisplayValue: "+e.getMessage(),e);
		}
		return result;
	}
	
	
	
}