/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.impl.jndi;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.List;

import javax.naming.Context;
import javax.naming.Name;
import javax.naming.RefAddr;
import javax.naming.Reference;
import javax.naming.spi.ObjectFactory;

import net.forwardfire.vasc.backend.VascBackendController;
import net.forwardfire.vasc.backend.VascBackendControllerLocal;
import net.forwardfire.vasc.backend.crud.VascBackendCrud;
import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascControllerLocal;
import net.forwardfire.vasc.core.VascControllerProvider;
import net.forwardfire.vasc.core.VascEntry;
import net.forwardfire.vasc.core.VascEntryConfigController;
import net.forwardfire.vasc.core.VascEntryController;
import net.forwardfire.vasc.core.VascEntryControllerLocal;
import net.forwardfire.vasc.core.VascEntryFieldTypeController;
import net.forwardfire.vasc.core.VascEntryGroup;
import net.forwardfire.vasc.core.VascEventChannelController;
import net.forwardfire.vasc.impl.DefaultVascFactory;

/**
 * JndiVascControllerFactory puts an new VascController into the jndi context.
 * note: needs to be flags as singleton resource !
 * 
 * TODO: this needs to tested on real tomcat/ee env.
 * 
 * @author Willem Cazander
 * @version 1.0 May 18, 2012
 */
public class JndiVascControllerFactory implements ObjectFactory  {

	private VascController vascController = null;
	private final static String CONF_PROVIDER = "vascControllerProvider"; 
	private final static String CONF_REMOVE_LOCAL = "removeLocal";
	
	/**
	 * @see javax.naming.spi.ObjectFactory#getObjectInstance(java.lang.Object, javax.naming.Name, javax.naming.Context, java.util.Hashtable)
	 */
	@Override
	public Object getObjectInstance(Object obj, Name name2, Context nameCtx,Hashtable<?, ?> env) throws Exception {
		
		if (vascController!=null) {
			return vascController;
		}
		
		// Fetch config parameters of this factory instance
		String configProviderClass = null;
		String configRemoveLocal = null;
		Reference ref = (Reference) obj;
		Enumeration<?> addrs = ref.getAll();
		while (addrs.hasMoreElements()) {
			RefAddr addr = (RefAddr) addrs.nextElement();
			String name = addr.getType();
			String value = (String) addr.getContent();
			if (CONF_PROVIDER.equals(name)) {
				configProviderClass = value;
			} else if (CONF_REMOVE_LOCAL.equals(name)) {
				configRemoveLocal = value;
			}
		}
		
		// optional config via provider interface.
		if (configProviderClass!=null && configProviderClass.isEmpty()==false) {
			ClassLoader cl = Thread.currentThread().getContextClassLoader();
			if (cl==null) {
				cl = this.getClass().getClassLoader();
			}
			Class<?> prioviderClass = cl.loadClass(configProviderClass);
			VascControllerProvider provider = (VascControllerProvider)prioviderClass.newInstance();
			vascController = provider.getVascController();
		}
		
		// Else create default
		if (vascController==null) {
			vascController = DefaultVascFactory.getDefaultVascController();
		}
		
		
		// Remove all local interfaces of vasc beans so no changes of config are possible.
		if (configRemoveLocal!=null && configRemoveLocal.isEmpty()==false && configRemoveLocal.equalsIgnoreCase("true")) {
			removeLocalInterfaces();
		}
		
		return vascController;
	}
	
	/**
	 * Wrap all local interfaces with non local interfaces.
	 */
	private void removeLocalInterfaces() {
		if ((vascController instanceof VascControllerLocal)==false) {
			return;
		}
		final VascControllerLocal vc = (VascControllerLocal)vascController;
		if (vc.getVascBackendController() instanceof VascBackendControllerLocal) {
			final VascBackendController backendController = vc.getVascBackendController();
			vc.setVascBackendController(new VascBackendController() {
				@Override
				public List<String> getVascBackendIds() {
					return backendController.getVascBackendIds();
				}
				@Override
				public VascBackendCrud<Serializable> getVascBackendById(String id) {
					return backendController.getVascBackendById(id);
				}
			});
		}
		if (vc.getVascEntryController() instanceof VascEntryControllerLocal) {
			final VascEntryController entryController = vc.getVascEntryController();
			vc.setVascEntryController(new VascEntryController() {
				@Override
				public List<String> getVascEntryIds() {
					return entryController.getVascEntryIds();
				}
				@Override
				public VascEntry getVascEntryById(String id) {
					return entryController.getVascEntryById(id);
				}
				@Override
				public List<String> getVascEntryByGroupId(String groupId) {
					return entryController.getVascEntryByGroupId(groupId);
				}
				@Override
				public VascEntryGroup getVascEntryGroupById(String id) {
					return entryController.getVascEntryGroupById(id);
				}
				@Override
				public List<String> getVascEntryGroupIds() {
					return entryController.getVascEntryGroupIds();
				}
			});
		}
		
		// TODO etc
		
		// Also wrap controller to non-local
		vascController = new VascController() {
			@Override
			public VascEventChannelController getVascEventChannelController() {
				return vc.getVascEventChannelController();
			}
			@Override
			public VascEntryFieldTypeController getVascEntryFieldTypeController() {
				return vc.getVascEntryFieldTypeController();
			}
			@Override
			public VascEntryController getVascEntryController() {
				return vc.getVascEntryController();
			}
			@Override
			public VascEntryConfigController getVascEntryConfigController() {
				return vc.getVascEntryConfigController();
			}
			@Override
			public VascBackendController getVascBackendController() {
				return vc.getVascBackendController();
			}
		};
	}
}
