/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.validators;

import java.lang.annotation.Annotation;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

/**
 * Validates the url annotations
 * 
 * @author Willem Cazander
 * @version 1.0 Sep 5, 2008
 */
public class VascObjectUrlValidator implements VascValidator {
	
	private static final long serialVersionUID = -1539950641644630956L;
	private Pattern pattern = null;
	
	/**
	 * Sets the regulare expression on which this object filters an String.<br>
	 * <br>
	 * When set to <code>Null</code> matching is disabled.<br>
	 * 
	 * @param regex	The regulare expression to match to.
	 */
	public void setForbidRegEx(String regex) {
		pattern = Pattern.compile(regex);
	}
	/**
	 * Returns the current regulare expression. 
	 * @return	The regulare expression.
	 */
	public String getForbidRegEx() {
		if(pattern==null) {
			return null;
		}
		return pattern.pattern();
	}
	
	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#isObjectValid(java.lang.Object)
	 */
	@Override
	public boolean isObjectValid(Object object) throws VascValidatorException {
		
		if (object==null) {
			return true;
		}
		URL url = null;
		if (object instanceof URL) {
			url = (URL)object;
		}
		if (object instanceof String) {
			try {
				url = new URL((String)object);
			} catch (MalformedURLException e) {
				throw new VascValidatorException("Url not valid: "+(String)object,e);
			}
		}
		if (url==null) {
			throw new VascValidatorException("Object could not be translated into url: "+object);
		}
		if (pattern==null) {
			return true;
		}
		String urlString = url.toExternalForm();
		//	test if regex matches
		Matcher m = pattern.matcher(urlString);
		return !m.find(); // if found then url it NOT valid
	}
	
	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#getAnnotationType()
	 */
	@Override
	public Class<?> getAnnotationType() {
		return VascObjectNull.class;
	}

	/**
	 * @see net.forwardfire.vasc.validators.VascValidator#initAnnotation(java.lang.annotation.Annotation)
	 */
	@Override
	public void initAnnotation(Annotation annotation) {
		VascObjectUrl a = VascObjectUrl.class.cast(annotation);
		setForbidRegEx(a.forbidRegex());
	}
	
	@Override
	public VascValidator clone() throws CloneNotSupportedException {
		VascObjectUrlValidator result = new VascObjectUrlValidator();
		result.pattern=pattern;
		return result;
	}
}