/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.tomcat;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.math.BigInteger;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.naming.NamingException;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascEntryControllerLocal;
import net.forwardfire.vasc.core.VascEventControllerType;
import net.forwardfire.vasc.impl.DefaultVascFactory;
import net.forwardfire.vasc.impl.x4o.VascDriver;

import org.apache.catalina.Server;
import org.apache.naming.ContextBindings;
import org.x4o.xml.io.X4OReader;

/**
 * VascDeployService parses "deploy/*.xml" automaticly for hotdeployments. 
 * 
 * @author Willem Cazander
 * @version 1.0 May 10, 2012
 */
public class JndiVascDeployer {

	private Server server = null;
	private VascController vascController = null;
	private Logger logger = null;
	private File deployDir = null;
	private int scanPeriod = 3;
	private AutoDeployManager autoDeployManager = null;
	private Map<File,String> fileCheckSums = null;
	
	public JndiVascDeployer() {
		logger = Logger.getLogger(JndiVascDeployer.class.getName());
		fileCheckSums = new HashMap<File,String>(20);
	}
	
	public void start() {
		if (vascController==null) {
			throw new NullPointerException("Can't deploy with null vascController.");
		}
		if (server==null) {
			throw new NullPointerException("Can't deploy with null server.");
		}
		if (deployDir==null) {
			throw new NullPointerException("Can't deploy with null deployDir.");
		}
		if (deployDir.exists()==false) {
			throw new IllegalStateException("Can't deploy with non-existing deployDir.");
		}
		if (deployDir.isDirectory()==false) {
			throw new IllegalStateException("Can't deploy with non-directory deployDir.");
		}
		
		// Start scan thread for auto (re)deployments
		Thread scanThread = new Thread(new AutoDeployManager());
		scanThread.setName("hotdeploy-scanner");
		scanThread.start();
	}
	
	public void stop() {
		if (autoDeployManager==null) {
			return;
		}
		autoDeployManager.stop();
	}
	

	public String createMd5Sum(File file) throws IOException, NoSuchAlgorithmException {
		FileInputStream in = new FileInputStream(file.getAbsolutePath());
		try {
			byte[] b = new byte[1024 * 64];
			int num = 0;
			MessageDigest md = MessageDigest.getInstance("MD5");
			while ((num = in.read(b)) != -1) {
				md.update(b, 0, num);
			}
			byte[] hashBytes = md.digest();
			BigInteger hashResult = new BigInteger(hashBytes);
			String hashString = hashResult.toString(16);
			return hashString;
		} finally {
			if (in != null) {
				in.close();
			}
		}
	}
	
	private void hotDeployVasc() throws NoSuchAlgorithmException, IOException {
		int deployed = 0;
		long startTime = System.currentTimeMillis();
		for (File file:deployDir.listFiles()) {
			if (file.canRead()==false) {
				continue;
			}
			if (file.getName().endsWith("xml")==false) {
				continue;
			}
			String md5File = createMd5Sum(file);
			String md5Deploy = fileCheckSums.get(file);
			if (md5Deploy!=null && md5Deploy.equals(md5File)) {
				continue;
			}
			fileCheckSums.put(file, md5File);
			deployed++;
			parseVascFile(file);
		}
		if (deployed > 0) {
			long stopTime = System.currentTimeMillis();
			logger.info("Done deploying total files read: "+deployed+" in "+(stopTime-startTime)+" ms.");
		}
	}
	
	public void parseVascFile(File file) {
		logger.info("Vasc open file: "+file.getAbsoluteFile());
		try {
			VascDriver driver = VascDriver.getInstance();
			X4OReader<VascController> reader = driver.createReader();
			driver.addVascController(reader,vascController);

			//File f = File.createTempFile("test-vasc", ".xml");
			//parser.setDebugOutputStream(new FileOutputStream(f));
			
			vascController.getVascEventChannelController().fireEvent(VascEventControllerType.LOAD_ENTRIES_BEFORE, vascController);
			reader.readFile(file);
			DefaultVascFactory.fillVascControllerLocalEntries((VascEntryControllerLocal) vascController.getVascEntryController(), vascController);
			vascController.getVascEventChannelController().fireEvent(VascEventControllerType.LOAD_ENTRIES_AFTER, vascController);
			
			
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			//fireChangeEvent();
		}
	}
	
	protected class AutoDeployManager implements Runnable {
		private volatile boolean run = true; 
		public void run() {
			try {
				Thread.sleep(2000); // let gui+tomcat start
				logger.info("AutoDeployManager started");
				
				//Server server = VascTechDemoStartup.getInstance().getTomcatService().getServer();
				Object token = "secretToken";
				String bindName = "autoDeployThread";
				try {
					ContextBindings.bindContext(bindName, server.getGlobalNamingContext(),token);
					ContextBindings.bindThread(bindName,token);
				} catch (NamingException e) {
					e.printStackTrace();
				}
				
				while(run) {
					try {
						hotDeployVasc();
					} catch (Exception e) {
						logger.log(Level.WARNING,"Error while depoying: "+e.getMessage(),e);
					}
					
					if (scanPeriod == 0) {
						scanPeriod = 1;
					}
					try {
						Thread.sleep(1000*scanPeriod);
					} catch (InterruptedException ie) {
						logger.info("Interrupted sleep");
						break;
					}
				}
				
				ContextBindings.unbindThread(bindName,token);
				ContextBindings.unbindContext(bindName,token);
				
				
			} catch (Exception e) {
				logger.log(Level.SEVERE,"Error in run: "+e.getMessage(),e);
			} finally {
				logger.info("AutoDeployManager stoped");
			}
		}
		
		public void stop() {
			run = false;
			this.notify();
		}
	}
	
	/**
	 * @return the deployDir
	 */
	public File getDeployDir() {
		return deployDir;
	}

	/**
	 * @param deployDir the deployDir to set
	 */
	public void setDeployDir(File deployDir) {
		this.deployDir = deployDir;
	}

	/**
	 * @return the scanPeriod
	 */
	public int getScanPeriod() {
		return scanPeriod;
	}

	/**
	 * @param scanPeriod the scanPeriod to set
	 */
	public void setScanPeriod(int scanPeriod) {
		this.scanPeriod = scanPeriod;
	}

	/**
	 * @return the server
	 */
	public Server getServer() {
		return server;
	}

	/**
	 * @param server the server to set
	 */
	public void setServer(Server server) {
		this.server = server;
	}

	/**
	 * @return the vascController
	 */
	public VascController getVascController() {
		return vascController;
	}

	/**
	 * @param vascController the vascController to set
	 */
	public void setVascController(VascController vascController) {
		this.vascController = vascController;
	}
}
