/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.tomcat;

import java.net.UnknownHostException;
import java.security.Principal;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Logger;

import org.apache.catalina.LifecycleException;
import org.apache.catalina.realm.GenericPrincipal;
import org.apache.catalina.realm.RealmBase;

import com.mongodb.BasicDBObject;
import com.mongodb.DB;
import com.mongodb.DBCollection;
import com.mongodb.DBCursor;
import com.mongodb.DBObject;
import com.mongodb.Mongo;
import com.mongodb.MongoOptions;
import com.mongodb.ServerAddress;

/**
 * MongoRealm does auth to mongo collections.
 * 
 * @author Willem Cazander
 * @version 1.0 May 13, 2012
 */
public class MongoRealm extends RealmBase {
	
	private Logger logger = Logger.getLogger(MongoRealm.class.getName());
	private String hostname = "localhost";
	private int port = 27017;
	private String database = null;
	private String authUser = null;
	private String authPass = null;
	private String userCollection = null;
	private String userField = "username";
	private String userPassField = "password";
	private String roleCollection = null;
	private String roleField = "role";
	private String roleUserField = userField;
	private boolean roleUserIdRef = false;
	private Mongo mongo = null;
	private DB db = null;
	
	@Override
	protected String getName() {
		return this.getClass().getSimpleName();
	}

	@Override
	protected String getPassword(String username) {
		DBObject user = getUser(username);
		if (user==null) {
			return null;
		}
		String password = (String) user.get(getUserPassField());
		return password;
	}
	
	protected DBObject getUser(String username) {
		DBCollection coll = db.getCollection(getUserCollection());
		DBObject query = new BasicDBObject();
		query.put(getUserField(), username);
		DBCursor cur = coll.find(query);
		if (cur.hasNext()==false) {
			return null;
		}
		DBObject user = cur.next();
		return user;
	}
	 
	@Override
	protected Principal getPrincipal(String username) {
		String password = getPassword(username);
		if (password==null) {
			return null;
		}
		DBCollection coll = db.getCollection(getRoleCollection());
		DBObject query = new BasicDBObject();
		if (isRoleUserIdRef()) {
			DBObject user = getUser(username);
			query.put(getRoleUserField(), user.get("_id"));
		} else {
			query.put(getRoleUserField(), username);
		}
		DBCursor cur = coll.find(query);
		if (cur.hasNext()==false) {
			return null;
		}
		List<String> roles = new ArrayList<String>(30);
		while (cur.hasNext()) {
			DBObject row = cur.next();
			String role = (String)row.get(getRoleField());
			roles.add(role);
		}
		return new GenericPrincipal(username, password, roles);
	}
	
	/**
	 * @see org.apache.catalina.realm.RealmBase#startInternal()
	 */
	@Override
	protected void startInternal() throws LifecycleException {
		super.startInternal();
		if (database==null) {
			throw new LifecycleException("database is null.");
		}
		if (userCollection==null) {
			throw new LifecycleException("userCollection is null.");
		}
		if (roleCollection==null) {
			throw new LifecycleException("roleCollection is null.");
		}
		createMongoConnection();
		logger.fine("Started mongo realm to database: "+getDatabase());
	}

	/**
	 * @see org.apache.catalina.realm.RealmBase#stopInternal()
	 */
	@Override
	protected void stopInternal() throws LifecycleException {
		super.stopInternal();
		if (mongo==null) {
			return;
		}
		mongo.close();
		logger.fine("Stopped mongo realm.");
	}
	
	private void createMongoConnection() throws LifecycleException  {
		ServerAddress server;
		try {
			server = new ServerAddress(hostname,port);
		} catch (UnknownHostException e) {
			throw new LifecycleException(e);
		}
		MongoOptions options = new MongoOptions();
		mongo = new Mongo(server,options);
		db = mongo.getDB(database);
		if (authUser!=null && authUser.isEmpty()==false && authPass!=null && authPass.isEmpty()==false) {
			boolean auth = db.authenticate(authUser, authPass.toCharArray());
			authPass = null;
			if (auth==false) {
				throw new LifecycleException("Could not auth to db: "+database+" with username: "+authUser);
			}
			
		}
		db.setReadOnly(true);
		logger.finer("Connection to: "+db.getName());
	}

	/**
	 * @return the hostname
	 */
	public String getHostname() {
		return hostname;
	}

	/**
	 * @param hostname the hostname to set
	 */
	public void setHostname(String hostname) {
		this.hostname = hostname;
	}

	/**
	 * @return the port
	 */
	public int getPort() {
		return port;
	}

	/**
	 * @param port the port to set
	 */
	public void setPort(int port) {
		this.port = port;
	}

	/**
	 * @return the database
	 */
	public String getDatabase() {
		return database;
	}

	/**
	 * @param database the database to set
	 */
	public void setDatabase(String database) {
		this.database = database;
	}

	/**
	 * @return the authUser
	 */
	public String getAuthUser() {
		return authUser;
	}

	/**
	 * @param authUser the authUser to set
	 */
	public void setAuthUser(String authUser) {
		this.authUser = authUser;
	}

	/**
	 * @return the authPass
	 */
	public String getAuthPass() {
		return authPass;
	}

	/**
	 * @param authPass the authPass to set
	 */
	public void setAuthPass(String authPass) {
		this.authPass = authPass;
	}

	/**
	 * @return the userCollection
	 */
	public String getUserCollection() {
		return userCollection;
	}

	/**
	 * @param userCollection the userCollection to set
	 */
	public void setUserCollection(String userCollection) {
		this.userCollection = userCollection;
	}

	/**
	 * @return the userField
	 */
	public String getUserField() {
		return userField;
	}

	/**
	 * @param userField the userField to set
	 */
	public void setUserField(String userField) {
		this.userField = userField;
	}

	/**
	 * @return the roleCollection
	 */
	public String getRoleCollection() {
		return roleCollection;
	}

	/**
	 * @param roleCollection the roleCollection to set
	 */
	public void setRoleCollection(String roleCollection) {
		this.roleCollection = roleCollection;
	}

	/**
	 * @return the roleField
	 */
	public String getRoleField() {
		return roleField;
	}

	/**
	 * @param roleField the roleField to set
	 */
	public void setRoleField(String roleField) {
		this.roleField = roleField;
	}

	/**
	 * @return the roleUserField
	 */
	public String getRoleUserField() {
		return roleUserField;
	}

	/**
	 * @param roleUserField the roleUserField to set
	 */
	public void setRoleUserField(String roleUserField) {
		this.roleUserField = roleUserField;
	}

	/**
	 * @return the userPassField
	 */
	public String getUserPassField() {
		return userPassField;
	}

	/**
	 * @param userPassField the userPassField to set
	 */
	public void setUserPassField(String userPassField) {
		this.userPassField = userPassField;
	}

	/**
	 * @return the roleUserIdRef
	 */
	public boolean isRoleUserIdRef() {
		return roleUserIdRef;
	}

	/**
	 * @param roleUserIdRef the roleUserIdRef to set
	 */
	public void setRoleUserIdRef(boolean roleUserIdRef) {
		this.roleUserIdRef = roleUserIdRef;
	}
}  