/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.demo.server.ui;

import java.awt.AWTException;
import java.awt.Color;
import java.awt.Cursor;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.MenuItem;
import java.awt.PopupMenu;
import java.awt.SystemTray;
import java.awt.TrayIcon;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.ArrayList;
import java.util.EventObject;
import java.util.List;
import java.util.Properties;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JOptionPane;
import javax.swing.UIManager;

import net.forwardfire.vasc.demo.server.core.VascTechDemoStartup;

import org.jdesktop.application.FrameView;
import org.jdesktop.application.SingleFrameApplication;

/**
 * SwingGuiService Shows the demo swing gui and vasc swing frontend.
 * 
 * @author Willem Cazander
 * @version 1.0 May 19, 2012
 */
public class ServerGuiApplication extends SingleFrameApplication {
	
	private Logger logger = null;
	private JStatusPanel statusPanel = null;
	private ImageIcon serverIcon = null;
	private UpdateInfoTask updateInfoTask = null;
	
	public ServerGuiApplication() {
		logger = Logger.getLogger(ServerGuiApplication.class.getName());
	}
	
	protected void startup() {
		installColorsLaF();
		addExitListener(new CloseWindowExitListener());
		serverIcon = createImageIcon("/net/forwardfire/vasc/demo/server/ui/resources/icon.png", "Vasc Icon");
		statusPanel = new JStatusPanel();
		FrameView mainView = getMainView();
		mainView.setComponent(statusPanel);
		mainView.getFrame().setMinimumSize(new Dimension(640,480));
		mainView.getFrame().setMaximumSize(new Dimension(800,600));
		mainView.getFrame().setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
		
		show(mainView);
		startSystemTray();
		
		updateInfoTask = new UpdateInfoTask();
		Thread t = new Thread(updateInfoTask);
		t.setName(UpdateInfoTask.class.getSimpleName());
		t.start();
	}
	
	class UpdateInfoTask implements Runnable {
		volatile public boolean run = true;
		volatile public boolean doUpdate = true;
		@Override
		public void run() {
			logger.finest("Updating thread started.");
			while(run) {
				try {
					Thread.sleep(1000);
					if (doUpdate) {
						statusPanel.updateInfo();
					}
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
			logger.finest("Updating thread stopped.");
		}
	}
	
	class CloseWindowExitListener implements ExitListener {
		@Override
		public boolean canExit(EventObject event) {
			if (event!=null && event.getSource().equals(ServerGuiApplication.this)) {
				updateInfoTask.run=false; // app will exit so stop update
				return true;
			} else {
				logger.info("Closing application window.");
				updateInfoTask.doUpdate=false; // close window 
				ServerGuiApplication.getInstance().getMainFrame().setVisible(false);
				return false;
			}
		}
		
		@Override
		public void willExit(EventObject event) {
		}
	}
	
	public void startupDone() {
		try {
			statusPanel.startupDone();
		} finally {
			getMainFrame().setCursor(Cursor.getDefaultCursor());
		}
	}
	
	/**
	 * Stop if requested from service
	 */
	public void stop() {
		//shutdown();
		exit(new EventObject(this));
	}
	
	public ImageIcon getServerIcon() {
		return serverIcon;
	}
	
	static public ServerGuiApplication getInstance() {
		return getInstance(ServerGuiApplication.class);
	}
	
	private void startSystemTray() {
		if (!SystemTray.isSupported()) {
			return;
		}
		final PopupMenu popup = new PopupMenu();
		final TrayIcon trayIcon = new TrayIcon(createImageIcon("/net/forwardfire/vasc/demo/server/ui/resources/tray-icon.png", "tray icon").getImage());
		final SystemTray tray = SystemTray.getSystemTray();
		
		MenuItem aboutItem = new MenuItem("About");
		aboutItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				logger.finer("Systray show about.");
				JOptionPane.showMessageDialog(ServerGuiApplication.getInstance().getMainFrame(), "Vasc Demo Tech Server:\nIs build to test and demo different parts of the vasc package.", "About Vasc Demo", JOptionPane.PLAIN_MESSAGE);
			}
		});
		MenuItem statusItem = new MenuItem("Open Status");
		statusItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				logger.finer("Systray open status window.");
				updateInfoTask.doUpdate=true;
				getMainFrame().setVisible(true);
			}
		});
		MenuItem exitItem = new MenuItem("Exit");
		exitItem.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				logger.finer("Systray exit application.");
				VascTechDemoStartup.getInstance().shutdown();
			}
		});
		
		popup.add(exitItem);
		popup.addSeparator();
		popup.add(aboutItem);
		popup.add(statusItem);
		
		trayIcon.setPopupMenu(popup);
		
		try {
			tray.add(trayIcon);
		} catch (AWTException e) {
			logger.warning("TrayIcon could not be added.");
		}
	}
	
	protected static ImageIcon createImageIcon(String path, String description) {
		URL imageURL = ServerGuiApplication.class.getResource(path);
		if (imageURL == null) {
			throw new NullPointerException("Could not find resource: "+path);
		}
		return new ImageIcon(imageURL, description);
	}
	
	private String installColorsLaF() {
		UIManager.put("TabbedPane.font",                Font.decode("SansSerif-BOLD-12"));
		UIManager.put("TitledBorder.font",              Font.decode("SansSerif-BOLD-16"));
		UIManager.put("FireDial.font",                  Font.decode("SansSerif-9"));
		
		String colorName = "laf-colors";
		ClassLoader cl = Thread.currentThread().getContextClassLoader();
		if (cl==null) {
			cl = this.getClass().getClassLoader();
		}
		InputStream in = cl.getResourceAsStream("net/forwardfire/vasc/demo/server/ui/resources/"+colorName+".properties");
		if (in==null) {
			logger.warning("Color schema not found: "+colorName);
			return "unknown";
		}
		try {
			Properties p = new Properties();
			p.load(in);
			for (Object key:p.keySet()) {
				String value = p.getProperty(key.toString());
				Color colorValue = Color.decode(value);
				UIManager.put(key,colorValue);
			}
		} catch (IOException e) {
			logger.warning("Could not load color schema: "+colorName+" error: "+e.getMessage());
		} finally {
			try {
				in.close();
			} catch (IOException e) {
			}
		}
		
		// Invert focus painters
		List<Object> keys = new ArrayList<Object>(UIManager.getLookAndFeelDefaults().keySet());
		for (Object keyObj:keys) {
			if ((keyObj instanceof String)==false) {
				continue;
			}
			String key = (String)keyObj;
			
			if (key.endsWith("[Focused].backgroundPainter")==false & key.endsWith("[Focused].iconPainter")==false) {
				continue;
			}
			String preKey = key.substring(0,key.indexOf("["));
			String postKey = "backgroundPainter";
			if (key.contains("iconPainter")) {
				postKey = "iconPainter";
			}
			
			logger.finer("Flipping painters of key: "+preKey);
			
			Object focusPainter = UIManager.getLookAndFeelDefaults().get(preKey+"[Focused]."+postKey);
			Object mouseOverPainter = UIManager.getLookAndFeelDefaults().get(preKey+"[MouseOver]."+postKey);
			UIManager.getLookAndFeelDefaults().put(preKey+"[Focused]."+postKey,mouseOverPainter);
			UIManager.getLookAndFeelDefaults().put(preKey+"[MouseOver]."+postKey,focusPainter);
			
			if (key.contains("iconPainter")) {
				Object focusPainterSelected = UIManager.getLookAndFeelDefaults().get(preKey+"[Focused+Selected]."+postKey);
				Object mouseOverPainterSelected = UIManager.getLookAndFeelDefaults().get(preKey+"[MouseOver+Selected]."+postKey);
				UIManager.getLookAndFeelDefaults().put(preKey+"[Focused+Selected]."+postKey,mouseOverPainterSelected);
				UIManager.getLookAndFeelDefaults().put(preKey+"[MouseOver+Selected]."+postKey,focusPainterSelected);
			}
		}
		
		return colorName;
	}
}
