/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.export.generic;

import java.io.OutputStream;
import java.io.PrintWriter;
import java.io.Serializable;

import net.forwardfire.vasc.backend.VascBackendException;
import net.forwardfire.vasc.core.VascEntryField;

/**
 * VascEntryExporterCsv writes data to csv output format.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 30, 2007
 */
public class VascEntryExportWriterCsv extends AbstractVascEntryExportWriter {
	
	private static final long serialVersionUID = 2770924442917617161L;
	static public final String EXPORT_TYPE = "csv";
	private PrintWriter p = null;
	private CsvSeparator separatorChar = CsvSeparator.SEMICOLON;
	private CsvQuote quoteChar = CsvQuote.DOUBLE;
	private CsvLineEnd lineEnd = CsvLineEnd.LF;
	
	enum CsvSeparator {
		TAB("\t"),
		COMMA(","),
		DOT("."),
		COLON(":"),
		SEMICOLON(";"),
		EQUAL("="),
		EQUALS("==");
		final String charType;
		CsvSeparator(String charType) {
			this.charType=charType;
		}
		String toCharType() {
			return charType;
		}
	}
	
	enum CsvQuote {
		NONE(""),
		SINGLE("'"),
		DOUBLE("\"");
		
		final String charType;
		CsvQuote(String charType) {
			this.charType=charType;
		}
		String toCharType() {
			return charType;
		}
	}
	
	enum CsvLineEnd {
		CR("\r"),
		LF("\n"),
		CRLF("\r\n");
		
		final String charType;
		CsvLineEnd(String charType) {
			this.charType=charType;
		}
		String toCharType() {
			return charType;
		}
	}
	
	@Override
	protected void doExportInit(OutputStream out) {
		p = new PrintWriter(out);
		String separatorCharString = getVascEntryExport().getWriterParameter("separatorChar");
		if (separatorCharString!=null && separatorCharString.isEmpty()==false) {
			separatorChar = CsvSeparator.valueOf(separatorCharString);
		}
		String quoteCharString = getVascEntryExport().getWriterParameter("quoteChar");
		if (quoteCharString!=null && quoteCharString.isEmpty()==false) {
			quoteChar = CsvQuote.valueOf(quoteCharString);
		}
		String lineEndString = getVascEntryExport().getWriterParameter("lineEnd");
		if (lineEndString!=null && lineEndString.isEmpty()==false) {
			lineEnd = CsvLineEnd.valueOf(lineEndString);
		}
	}

	@Override
	protected void doExportStart() {
		//p.write("# csv\n");
		for (VascEntryField c:getVascEntry().getVascEntryFields()) {
			p.write(c.getId()+separatorChar.toCharType());
		}
		p.write(lineEnd.toCharType());
	}

	@Override
	protected void doExportEnd() {
		p.write("#EOF end-of-file");
		p.write(lineEnd.toCharType());
		p.flush();
	}

	@Override
	protected void doExportRowStart(Serializable row) {
	}

	@Override
	protected void doExportRowEnd(Serializable row,boolean isLast) {
		p.write(lineEnd.toCharType());
		p.flush();
	}

	@Override
	protected void doExportRowField(Serializable o, VascEntryField c,boolean isLast) throws VascBackendException {
		p.write(quoteChar.toCharType());
		p.write(""+c.getVascEntryFieldValue().getValue(c.getBackendName(), o));
		p.write(quoteChar.toCharType());
		if (isLast==false) {
			p.write(separatorChar.toCharType());
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryExport#getMineType()
	 */
	public String getMineType() {
		return "text/csv";
	}

	/**
	 * @see net.forwardfire.vasc.core.entry.VascEntryExport#getFileType()
	 */
	public String getFileType() {
		return EXPORT_TYPE;
	}
}