/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.frontend.web.export;

import java.io.IOException;
import java.io.OutputStream;
import java.util.logging.Logger;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import net.forwardfire.vasc.core.VascController;
import net.forwardfire.vasc.core.VascControllerProvider;
import net.forwardfire.vasc.core.VascEntryLocal;
import net.forwardfire.vasc.core.entry.VascEntryExport;
import net.forwardfire.vasc.core.entry.VascEntryExportWriter;
import net.forwardfire.vasc.core.VascException;

/**
 * VascExportServlet exports the data from the vasc exporters on fixed url.
 * 
 * @author Willem Cazander
 * @version 1.0 May 12, 2012
 */
public class VascExportServlet extends HttpServlet {

	private Logger logger = Logger.getLogger(VascExportServlet.class.getName());
	private static final long serialVersionUID = 6165530418422144322L;
	private VascControllerProvider vascControllerProvider = null;
	private boolean forceTree = false;
	
	/**
	 * @see javax.servlet.GenericServlet#init()
	 */
	@Override
	public void init() throws ServletException {
		super.init();
		String className = getInitParameter("vascControllerProvider");
		if (className==null) {
			throw new ServletException("Can't start with null vascControllerProvider init parameter.");
		}
		String exportTreeStr = getInitParameter("forceTree");
		if (exportTreeStr!=null && "true".equalsIgnoreCase(exportTreeStr)) {
			forceTree = true;
		}
		String classArgument = null;
		int atIndex = className.indexOf('@');
		if (atIndex > 0) {
			classArgument = className.substring(0,atIndex);
			className = className.substring(atIndex+1);
		}
		try {
			ClassLoader cl = Thread.currentThread().getContextClassLoader();
			if (cl == null) {
				cl = className.getClass().getClassLoader();
			}
			Class<?> clazz = cl.loadClass(className);
			if (classArgument==null) {
				vascControllerProvider = (VascControllerProvider)clazz.newInstance();
			} else {
				vascControllerProvider = (VascControllerProvider)clazz.getConstructor(String.class).newInstance(classArgument);
			}
		} catch (Exception e) {
			throw new ServletException("Could not create VascControllerProvider: "+e.getMessage(),e);
		}
		logger.info("Export servlet started with forceTree: "+forceTree+" prov: "+classArgument+"@"+className);
	}

	/**
	 * @see javax.servlet.http.HttpServlet#service(javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
	 */
	@Override
	protected void service(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		String method = request.getMethod();
		if (method.equals("GET")==false) {
			response.sendError(HttpServletResponse.SC_METHOD_NOT_ALLOWED, "Only GET allowed.");
			return;
		}
		// Remove prefixes
		String path = request.getRequestURI();
		String servlet = request.getServletPath();
		if (path.contains(servlet)) {
			path  = path.substring(path.indexOf(servlet)+servlet.length());
		}
		if (path.startsWith("/")) {
			path = path.substring(1);
		}
		
		// stuff to fill
		String entryId = null;
		String exportId = null;
		
		int index = path.indexOf("/");
		if (forceTree && index<0) {
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "Only tree request allowed.");
			return;
		}
		
		if (index>0) {
			String actionString = null;
			if (index>0) {
				actionString = path.substring(index+1);
				entryId = path.substring(0,index);
			} else {
				entryId = path;
			}
			
			if (actionString!=null) {
				index = actionString.indexOf("/");
				String recordString = null;
				if (index>0) {
					recordString = actionString.substring(index+1);
					exportId = actionString.substring(0,index);  
				} else {
					exportId = actionString;	
				}
				if (recordString!=null) {
				//	actionRecordId = recordString;
				}
			}
		}
		
		// tree overrules parameters.
		if (entryId==null) {
			entryId = request.getParameter("ve");
		}
		if (exportId==null) {
			exportId = request.getParameter("et");
		}
		
		if (entryId==null) {
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "No entry given");
			return;
		}
		if (entryId.isEmpty()) {
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "No entry given");
			return;
		}
		if (exportId==null) {
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "No export type given");
			return;
		}
		if (exportId.isEmpty()) {
			response.sendError(HttpServletResponse.SC_BAD_REQUEST, "No export type given");
			return;
		}
		
		//out = new PrintWriter(new OutputStreamWriter(response.getOutputStream(), response.getCharacterEncoding()), true);
		OutputStream out = response.getOutputStream();
		try {
			VascController vc = vascControllerProvider.getVascController();
			VascEntryLocal ve = (VascEntryLocal)vc.getVascEntryConfigController().configVascEntry(vc, entryId);
			vc.getVascEntryConfigController().configVascFrontendController(vc, ve);
			
			ve.getVascFrontendController().getVascFrontendActions().refreshData();
			
			VascEntryExport ex = vc.getVascEntryConfigController().getVascEntryExporterById(exportId);
			VascEntryExportWriter exw = ex.createExportWriter();
			exw.doInit(ex, ve);
			
			String filename = entryId+"-export."+exw.getFileType();
			response.setHeader("Content-disposition", "attachment; filename=" + filename);
			String contentType = exw.getMineType();
			response.setContentType(contentType);
			
			exw.doExport(out);
		} catch (VascException e) {
			response.setStatus(500);
			response.setContentType("text/html");
		}
		out.flush();
		out.close();
	}
}
