/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.lib.i18n.config;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.logging.Logger;

/**
 * Loads and reads the configuration to load all resources.
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 8, 2012
 */
public class BundleConfigParser {
	
	private Logger logger = null;
	public static final String CONFIG_RESOURCE_DEFAULT = "META-INF/root-bundle.properties";
	protected static final String DEFAULT_CHAR_SET = "UTF-8";
	protected List<Locale> applicationSupportedLocales = null;
	protected List<BundleConfig> bundlesConfigs = null;
	protected List<String> bundleNames = null;
	protected Charset defaultCharSet = null;
		
	/**
	 * Creates new BundleConfigParser object.
	 */
	public BundleConfigParser() {
		logger = Logger.getLogger(BundleConfigParser.class.getName());
		applicationSupportedLocales = new ArrayList<Locale>(10);
		bundlesConfigs = new ArrayList<BundleConfig>(10);
		bundleNames = new ArrayList<String>(10);
		defaultCharSet = Charset.forName(DEFAULT_CHAR_SET);
	}
		
	/**
	 * Cleans config and 
	 */
	public void parseConfig() {
		cleanConfig();
		Properties properties = new Properties();
		readPropertiesConfig(properties);
		parseProperties(properties);
	}
	
	/**
	 * Returns List of supported locales by this root bundle config.
	 */
	public List<Locale> getApplicationSupportedLocales() {
		return applicationSupportedLocales;
	}
	
	/**
	 * Returns List of ordered bundle configs to load into root bundle.
	 */
	public List<BundleConfig> getBundleConfigs() {
		return bundlesConfigs;
	}
	
	/**
	 * Loads BundleConfig and returns localized data as map.
	 */
	public Map<String,String> loadBundleConfigData(BundleConfig bundle,Locale locale) {
		Properties properties = new Properties();
		try {
			switch (bundle.getUriType()) {
				default:
				case RESOURCE:
					readPropertiesBundle(properties,bundle,locale);
					break;
				case URL:
					readPropertiesBundleUrl(properties,bundle,new URL(bundle.getUri()));
					break;
				case FILE:
					readPropertiesBundleFile(properties,bundle,new File(bundle.getUri()));
					break;
			}
		} catch (Exception e) {
			if (bundle.isOptional()) {
				logger.info("Could not load optional bundle with uri: "+bundle.getUri()+" message: "+e.getMessage());
			} else {
				throw new RuntimeException(e);
			}
		}
		Map<String,String> result = new HashMap<String,String>(100);
		for (Object keyObject:properties.keySet()) {
			String key = (String)keyObject;
			if (bundle.getExclude()!=null && key.matches(bundle.getExclude())) {
				logger.finest("Excluding key: "+key+" from bundle: "+bundle.getName());
				continue;
			}
			String value = properties.getProperty(key);
			result.put(key,value);
		}
		logger.fine("Loaded "+bundle.getName()+" with total keys: "+result.size());
		return result;
	}
	
	/**
	 * Cleans bundle configs and supported locales.
	 */
	protected void cleanConfig() {
		bundlesConfigs.clear();
		applicationSupportedLocales.clear();		
	}
	
	/**
	 * Parsed the root-bundle.properties file format.
	 */
	protected void parseProperties(Properties p) {
		Map<String,BundleConfig> bundles = new HashMap<String,BundleConfig>(10);
		for (Object keyO:p.keySet()) {
			String keyFull = (String)keyO;
			String value = p.getProperty(keyFull);
			if (keyFull.contains(".")==false) {
				continue;
			}
			String keyConfig = keyFull.substring(0,keyFull.indexOf('.'));
			String keyField = keyFull.substring(keyFull.indexOf('.')+1,keyFull.length());
			
			if ("locale".equals(keyConfig) && "true".equalsIgnoreCase(value)) {
				applicationSupportedLocales.add(new Locale(keyField));
				continue;
			}
			if ("config".equals(keyConfig)) {
				if ("charset".equals(keyField)) {
					defaultCharSet = Charset.forName(value);
				} else if ("bundles".equals(keyField)) {
					String[] configBundles = value.split(",");
					bundleNames.addAll(Arrays.asList(configBundles));
				}
				continue;
			}
			
			BundleConfig c = bundles.get(keyConfig);
			if (c==null) {
				c = new BundleConfig();
				c.setName(keyConfig);
				bundles.put(keyConfig,c);
			}
			if ("uri".equals(keyField)) {
				c.setUri(value);
			} else if ("type".equals(keyField)) {
				c.setUriType(BundleURIType.valueOf(value));
			} else if ("exclude".equals(keyField)) {
				c.setExclude(value);
			} else if ("format".equals(keyField)) {
				c.setFormat(BundleFormat.valueOf(value));
			} else if ("optional".equals(keyField)) {
				c.setOptional(new Boolean(value));
			} else if ("charset".equals(keyField)) {
				c.setCharSet(Charset.forName(value));
			}
		}
		if (bundleNames.isEmpty()) {
			bundlesConfigs.addAll(bundles.values());
		} else {
			for (String bundleName:bundleNames) {
				BundleConfig bundle = bundles.get(bundleName);
				bundlesConfigs.add(bundle);
			}
		}
		for (BundleConfig bundle:bundlesConfigs) {
			if (bundle.getCharSet()==null) {
				bundle.setCharSet(defaultCharSet);
			}
		}
	}
	
	/**
	 * Reads properties file from disk.
	 */
	protected void readPropertiesBundleFile(Properties p,BundleConfig bundle,File file) {
		logger.fine("Reading bundle file: "+file.getAbsoluteFile());
		try {
			readProperties(p,new InputStreamReader(new FileInputStream(file),bundle.getCharSet()));
		} catch (Exception e) {
			throw new RuntimeException("Could not load resource file error: "+e.getMessage(),e);
		}
	}
	
	/**
	 * Reads properties file from url.
	 */
	protected void readPropertiesBundleUrl(Properties p,BundleConfig bundle,URL url) {
		logger.fine("Reading bundle url: "+url.toString());
		try {
			readProperties(p,new InputStreamReader(url.openStream(),bundle.getCharSet()));
		} catch (Exception e) {
			throw new RuntimeException("Could not load resource file error: "+e.getMessage(),e);
		}
	}
	
	/**
	 * Read ResourceBundle from classpath.
	 */
	protected void readPropertiesBundle(Properties p,BundleConfig bundle,Locale locale) {
		ResourceBundle bundleReal = ResourceBundle.getBundle(bundle.getUri(),locale,new BundleCharSetControl(bundle.getCharSet()));
		for (String key:bundleReal.keySet()) {
			String value = bundleReal.getString(key);
			p.put(key, value);
		}
	}
		
	/**
	 * Read properties from Reader and close Reader.
	 */
	protected void readProperties(Properties p,Reader in) {
		try {
			p.load(in);
		} catch (Exception e) {
			throw new RuntimeException("Could not load properties error: "+e.getMessage(),e);
		} finally {
			if (in!=null) {
				try {
					in.close();
				} catch (IOException e) {
				}
			}
		}
	}
	
	/**
	 * Read properties file from resource classpath.
	 */
	protected void readPropertiesConfig(Properties p) {
		try {
			ClassLoader cl = Thread.currentThread().getContextClassLoader();
			if (cl == null) {
				cl = p.getClass().getClassLoader(); // fallback
			}
			readProperties(p,new InputStreamReader(cl.getResourceAsStream(CONFIG_RESOURCE_DEFAULT),defaultCharSet));
		} catch (Exception e) {
			throw new RuntimeException("Could not load resource: "+CONFIG_RESOURCE_DEFAULT+" error: "+e.getMessage(),e);
		}
	}
}
