/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.lib.jr4o;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.OutputStream;
import java.util.HashMap;
import java.util.Map;

import org.x4o.xml.io.X4OConnectionException;
import org.x4o.xml.io.X4OReader;
import org.xml.sax.SAXException;

import net.forwardfire.vasc.lib.jr4o.data.JRDynamicDataSource;
import net.sf.jasperreports.engine.JRException;
import net.sf.jasperreports.engine.JRExporter;
import net.sf.jasperreports.engine.JRExporterParameter;
import net.sf.jasperreports.engine.JasperCompileManager;
import net.sf.jasperreports.engine.JasperFillManager;
import net.sf.jasperreports.engine.JasperPrint;
import net.sf.jasperreports.engine.JasperReport;
import net.sf.jasperreports.engine.design.JRDesignParameter;
import net.sf.jasperreports.engine.design.JasperDesign;
import net.sf.jasperreports.engine.export.JRCsvExporter;
import net.sf.jasperreports.engine.export.JRPdfExporter;
import net.sf.jasperreports.engine.export.JRRtfExporter;
import net.sf.jasperreports.engine.export.JRXlsExporter;
import net.sf.jasperreports.engine.export.JRXmlExporter;

/**
 * JRDesignManager can work with multiple designs for one JRDynamicDataSource.
 * 
 * @author Willem Cazander
 * @version 1.0 May 10, 2012
 */
public class JR4ODesignManager {

	private String resourceBundle = null;
	private Map<String,Object> parameters = null;
	private JRDynamicDataSource dataSource = null;
	private Map<String,JasperDesign> reportDesigns = null;
	//private Map<String,ReportCache> cachedReports = null;
	//private boolean cacheReports = true;
	
	public JR4ODesignManager(JRDynamicDataSource dataSource,String resourceBundle) {
		this(dataSource,resourceBundle,new HashMap<String,Object>(1));
	}
	
	public JR4ODesignManager(JRDynamicDataSource dataSource,String resourceBundle,Map<String,Object> parameters) {
		if (dataSource==null) {
			throw new NullPointerException("Can't create dynamic reports with null dynamic datasource.");
		}
		if (parameters==null) {
			throw new NullPointerException("Can't create dynamic reports with null parameters map.");
		}
		if (resourceBundle==null) {
			throw new NullPointerException("Can't create dynamic reports with null resourceBundle.");
		}
		this.dataSource = dataSource;
		this.parameters = parameters;
		this.resourceBundle = resourceBundle;
		reportDesigns = new HashMap<String,JasperDesign>(20);
	}
	
	
	public JasperDesign getJasperDesignByName(String designName) {
		if (designName==null) {
			throw new NullPointerException("Can't search for report with null name.");
		}
		return reportDesigns.get(designName);
	}
	
	public void addJasperDesign(JasperDesign design) {
		if (design==null) {
			throw new NullPointerException("Can add null design.");
		}
		if (design.getResourceBundle()==null) {
			design.setResourceBundle(resourceBundle);
		}
		reportDesigns.put(design.getName(), design);
	}
	
	private X4OReader<JR4ODesignRoot> createParser() {
		JR4ODriver driver = new JR4ODriver();
		X4OReader<JR4ODesignRoot> reader = driver.createReader();
		reader.addELBeanInstance(JR4ODriver.EL_DATASOURCE_NAME, dataSource);
		Map<String,Object> elBeans = dataSource.addDynamicELBean();
		for (String key:elBeans.keySet()) {
			Object value = elBeans.get(key);
			reader.addELBeanInstance(key, value);
		}
		return reader;
	}
	
	public void parseFile(File file) throws FileNotFoundException, X4OConnectionException, SAXException, IOException {
		if (file==null) {
			throw new NullPointerException("Can't parse null file.");
		}
		X4OReader<JR4ODesignRoot> parser = createParser();
		JR4ODesignRoot root = parser.readFile(file);
		for (JasperDesign design:root.getDesigns()) {
			addJasperDesign(design);
		}
	}
	
	public void parseResource(String resourceName) throws X4OConnectionException, SAXException, IOException {
		if (resourceName==null) {
			throw new NullPointerException("Can't parse null resouce.");
		}
		X4OReader<JR4ODesignRoot> parser = createParser();
		JR4ODesignRoot root = parser.readResource(resourceName);
		for (JasperDesign design:root.getDesigns()) {
			addJasperDesign(design);
		}
	}
	
	public JasperReport compileReport(String designName) throws JRException {
		JasperDesign design = getJasperDesignByName(designName);
		if (design==null) {
			throw new NullPointerException("Could not find design with name: "+designName);
		}
		return compileReport(design);
	}
	
	public JasperReport compileReport(JasperDesign design) throws JRException {
		if (design==null) {
			throw new NullPointerException("Can't compile with null design.");
		}
		for (String key:parameters.keySet()) {
			Object value = parameters.get(key);
			JRDesignParameter par = new JRDesignParameter();
			par.setName(key);
			par.setValueClass(value.getClass());
			design.addParameter(par);
		}
		dataSource.addDynamicColumnClassFields(design);
		return JasperCompileManager.compileReport(design);
	}
	
	public JasperPrint printReport(JasperReport report) throws JRException {
		return JasperFillManager.fillReport(report, parameters, dataSource);
	}
	
	private void saveReport(JasperPrint reportPrint,JRExportType type,boolean isFile,Object out) throws JRException {
		JRExporter exporter = null;
		switch (type) {
			default:
			case PDF:	exporter = new JRPdfExporter();		break;
			case RTF:	exporter = new JRRtfExporter();		break;
			case CSV:	exporter = new JRCsvExporter();		break;
			case XLS:	exporter = new JRXlsExporter();		break;
			case XML:	exporter = new JRXmlExporter();		break;
		}
		if (isFile) {
			exporter.setParameter(JRExporterParameter.OUTPUT_FILE_NAME,out);
		} else {
			exporter.setParameter(JRExporterParameter.OUTPUT_STREAM,out);
		}
		exporter.setParameter(JRExporterParameter.JASPER_PRINT,reportPrint);
		exporter.exportReport();
	}
	
	public void saveReportStream(JasperPrint reportPrint,JRExportType type,OutputStream out) throws JRException {
		saveReport(reportPrint,type,false,out);
	}
	
	public void saveReportFile(JasperPrint reportPrint,JRExportType type,String fileName) throws JRException {
		saveReport(reportPrint,type,true,fileName);
	}
	
	public void saveReportStream(String name,JRExportType type,OutputStream out) throws JRException {
		saveReportStream(printReport(compileReport(name)),type,out);
	}
	
	public void saveReportFile(String name,JRExportType type,String file) throws JRException {
		saveReportFile(printReport(compileReport(name)),type,file);
	}
	
	public enum JRExportType {
		PDF("text/pdf","pdf"),
		RTF("text/rtf","rtf"),
		CSV("text/csv","csv"),
		XLS("text/xls","xls"),
		XML("text/xml","xml");
		
		private String mineType = null;
		private String fileType = null;
		private JRExportType(String mineType,String fileType) {
			this.mineType=mineType;
			this.fileType=fileType;
		}
		public String getMineType() {
			return mineType;
		}
		public String getFileType() {
			return fileType;
		}
	}
}
