/*
 * Copyright 2007-2012 forwardfire.net All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, are permitted provided
 * that the following conditions are met:
 * 
 * * Redistributions of source code must retain the above copyright notice, this list of conditions and the
 *   following disclaimer.
 * * Redistributions in binary form must reproduce the above copyright notice, this list of conditions and
 *   the following disclaimer in the documentation and/or other materials provided with the distribution.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL
 * THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
 * TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package net.forwardfire.vasc.xpql.query;

import java.lang.reflect.Constructor;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.forwardfire.vasc.xpql.QueryStore;
import net.forwardfire.vasc.xpql.impl.DefaultQueryParameterValue;


/**
 * AbstractQuery
 * 
 * @author Willem Cazander
 * @version 1.0 Mar 4, 2007
 */
abstract public class AbstractQuery implements Query {

	private String name = null;
	private List<QueryPart> queryParts = null;
	private Map<String,QueryParameterValue> localQueryParameterValues = null;
	private Map<String,QueryParameterValue> queryParameterValues = null;
	private List<QueryParameterValue> orderQueryParameterValues = null;
	private List<String> queryComments = null;
	private Map<String,Object> properties = null;
	private QueryType type = null;
	private QueryStore queryStore = null;
	
	/**
	 * Init all fields.
	 */
	public AbstractQuery() {
		queryParts = new ArrayList<QueryPart>(10);
		localQueryParameterValues = new HashMap<String,QueryParameterValue>(6);
		queryParameterValues = new HashMap<String,QueryParameterValue>(6);
		orderQueryParameterValues = new ArrayList<QueryParameterValue>(6);
		queryComments = new ArrayList<String>(2);
		properties = new HashMap<String,Object>(5);
	}
	
	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getName()
	 */
	@Override
	public String getName() {
		return name;
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#setName(java.lang.String)
	 */
	@Override
	public void setName(String name) {
		this.name=name;
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#addQueryComment(java.lang.String)
	 */
	@Override
	public void addQueryComment(String comment) {
		queryComments.add(comment);
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#addQueryParameterValue(net.forwardfire.vasc.xpql.query.QueryParameterValue)
	 */
	@Override
	public void addQueryParameterValue(QueryParameterValue value) {
		queryParameterValues.put(value.getName(), value);
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#addLocalQueryParameterValue(net.forwardfire.vasc.xpql.query.QueryParameterValue)
	 */
	@Override
	public void addLocalQueryParameterValue(QueryParameterValue value) {
		localQueryParameterValues.put(value.getName(), value);
	}
	
	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#addOrderQueryParameterValue(net.forwardfire.vasc.xpql.query.QueryParameterValue)
	 */
	@Override
	public void addOrderQueryParameterValue(QueryParameterValue value) {
		orderQueryParameterValues.add(value);
	}
	
	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#addQueryPart(net.forwardfire.vasc.xpql.query.QueryPart)
	 */
	@Override
	public void addQueryPart(QueryPart queryPart) {
		queryParts.add(queryPart);
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getProperty(java.lang.String)
	 */
	@Override
	public Object getProperty(String name) {
		return properties.get(name);
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getPropertyString(java.lang.String)
	 */
	@Override
	public String getPropertyString(String name) {
		return getProperty(name).toString();
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getQueryComments()
	 */
	@Override
	public List<String> getQueryComments() {
		return queryComments;
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getQueryParameterValue(java.lang.String)
	 */
	@Override
	public QueryParameterValue getQueryParameterValue(String name) {
		return queryParameterValues.get(name);
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getQueryParts()
	 */
	@Override
	public List<QueryPart> getQueryParts() {
		return queryParts;
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#setProperty(java.lang.String, java.lang.Object)
	 */
	@Override
	public void setProperty(String name, Object value) {
		properties.put(name,value);
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getPropertyKeys()
	 */
	@Override
	public Collection<String> getPropertyKeys() {
		return properties.keySet();
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getQueryParameters()
	 */
	@Override
	public List<String> getQueryParameters() {
		return null;
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getQueryParameterValues()
	 */
	@Override
	public Collection<QueryParameterValue> getQueryParameterValues() {
		return queryParameterValues.values();
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getLocalQueryParameterValues()
	 */
	@Override
	public Collection<QueryParameterValue> getLocalQueryParameterValues() {
		return localQueryParameterValues.values();
	}
	
	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getOrderQueryParameterValues()
	 */
	@Override
	public List<QueryParameterValue> getOrderQueryParameterValues() {
		return orderQueryParameterValues;
	}
	
	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#setQueryParameter(java.lang.String, java.lang.Object)
	 */
	@Override
	public void setQueryParameter(String name, Object value) {
		if (name==null) {
			throw new NullPointerException("name can't be null.");
		}
		QueryParameterValue parameter = getQueryParameterValue(name);
		if (parameter==null) {
			// not found create new one
			parameter = new DefaultQueryParameterValue();
			parameter.setName(name);
			parameter.setType(QueryParameterValue.QueryParameterType.parameter);
			addQueryParameterValue(parameter);
		}
		
		// check if we need to convert
		//System.out.println("Con: "+parameter.getValueType()+" value: "+value+" valueClass: "+value.getClass());
		if (parameter.getValueType()!=null && value instanceof String) {
			try {
				Constructor<?> c = parameter.getValueType().getConstructor(String.class);
				value = c.newInstance(value);
			} catch (Exception e) {
				// TODO: fix
				throw new RuntimeException(e);
			}
		}	
		parameter.setValue(value);
	}

	@Override
	public void setQueryParameters(Map<String,Object> parameters) {
		for (String key:parameters.keySet()) {
			setQueryParameter(key,parameters.get(key));
		}
	}
	
	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getType()
	 */
	@Override
	public QueryType getType() {
		return type;
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#setType(net.forwardfire.vasc.xpql.query.Query.QueryType)
	 */
	@Override
	public void setType(QueryType type) {
		this.type=type;
	}	
	
	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#getQueryStore()
	 */
	@Override
	public QueryStore getQueryStore() {
		return queryStore;
	}

	/**
	 * @see net.forwardfire.vasc.xpql.query.Query#setQueryStore(net.forwardfire.vasc.xpql.query.QueryStore)
	 */
	@Override
	public void setQueryStore(QueryStore store) {
		this.queryStore=store;
	}

	@Override
	abstract public Query clone() throws CloneNotSupportedException;
}